#ifndef FLASH_LAYOUT_H
#define FLASH_LAYOUT_H

//----------------------------------------------------------------------------
// NAND customization
//----------------------------------------------------------------------------
#ifdef CC_USE_256MB_NAND
    /* Non-IPTV */
    #define NAND_FLASH_TOTAL_SIZE           (256*1024*1024)     /* 256 MB */

    #ifdef CC_256MB_NAND_SUPPORT_PKG_DOWNLOAD
        /* Non-IPTV with PKG download (for OAD in EU, PAD) */
        #define NAND_FLASH_3RD_RW_SIZE      (90*1024*1024)      /*  90 MB. Max PKG size:78MB */

        #ifdef REPLACE_EEPROM_WITH_FLASH
        #define NAND_FLASH_3RD_SIZE         (20*1024*1024)      /*  40 MB - 20 MB(virtual eeprom) */
        #else
        #define NAND_FLASH_3RD_SIZE         (40*1024*1024)      /*  40 MB */
        #endif

    #else
        /* Non-IPTV without PKG download (ex: US, SA, CH, GA, etc) */
        #define NAND_FLASH_3RD_SIZE         (95*1024*1024)      /*  95 MB */

        #ifdef REPLACE_EEPROM_WITH_FLASH
        #define NAND_FLASH_3RD_RW_SIZE      (15*1024*1024)      /*  35 MB - 20 MB(virtual eeprom) */
        #else
        #define NAND_FLASH_3RD_RW_SIZE      (35*1024*1024)      /*  35 MB */
        #endif
    #endif

    #ifdef CC_DISABLE_BASIC_PART
        #define NAND_FLASH_ROOTFS_SIZE      (45*1024*1024)      /*  45 MB */ /* Normal behavior */
        #define NAND_FLASH_APPIMAGE_SIZE    (256*1024)          /* 256 KB */ /* Normal behavior */
    #else
        #define NAND_FLASH_ROOTFS_SIZE      (8*1024*1024)       /*   8 MB */ /* Debug+Development purpose */
        #define NAND_FLASH_APPIMAGE_SIZE    (40*1024*1024)      /*  40 MB */ /* Debug+Development purpose */
    #endif
#else
    /* IPTV */
    #define NAND_FLASH_TOTAL_SIZE           (512*1024*1024)     /* 512 MB */
	
    /* A temporary change for 3rd size is not enough in cli pkg */
    #ifdef CC_ENLARGE_3RD_SIZE	
        #define NAND_FLASH_3RD_SIZE 			(228*1024*1024) 	/* 128 MB */

        #ifdef REPLACE_EEPROM_WITH_FLASH
        #define NAND_FLASH_3RD_RW_SIZE			(118*1024*1024) 	/* 138 MB - 20 MB(virtual eeprom) */
		#else
		#define NAND_FLASH_3RD_RW_SIZE			(138*1024*1024) 	/* 138 MB */
		#endif /*REPLACE_EEPROM_WITH_FLASH*/

    #else
        #define NAND_FLASH_3RD_SIZE 			(128*1024*1024) 	/* 128 MB */

    #ifdef CC_ENLARGE_APPIMAGE_SIZE
        #ifdef REPLACE_EEPROM_WITH_FLASH
        #define NAND_FLASH_3RD_RW_SIZE          (200*1024*1024)     /* 200 MB - 20 MB(virtual eeprom) */
        #else
        #define NAND_FLASH_3RD_RW_SIZE          (220*1024*1024)     /* 210 MB */
        #endif /*REPLACE_EEPROM_WITH_FLASH*/    
    #else    
        #ifdef REPLACE_EEPROM_WITH_FLASH
        #define NAND_FLASH_3RD_RW_SIZE			(218*1024*1024) 	/* 218 MB - 20 MB(virtual eeprom) */
        #else
        #define NAND_FLASH_3RD_RW_SIZE			(238*1024*1024) 	/* 238 MB */
        #endif /*REPLACE_EEPROM_WITH_FLASH*/	
    #endif
    #endif /*CC_ENLARGE_3RD_SIZE*/

    #ifdef CC_DISABLE_BASIC_PART
    #define NAND_FLASH_ROOTFS_SIZE          (52*1024*1024)      /*  52 MB */ /* Normal behavior */
    #define NAND_FLASH_APPIMAGE_SIZE        (256*1024)          /* 256 KB */ /* Normal behavior */
    #else
        #define NAND_FLASH_ROOTFS_SIZE          (8*1024*1024)       /*   8 MB */ /* Debug+Development purpose */
        #ifdef CC_ENLARGE_APPIMAGE_SIZE
        #define NAND_FLASH_APPIMAGE_SIZE        (70*1024*1024)      /*  70 MB */ /* Debug+Development purpose */
        #else
        #define NAND_FLASH_APPIMAGE_SIZE        (52*1024*1024)      /*  52 MB */ /* Debug+Development purpose */
        #endif
    #endif
#endif

#define NAND_FLASH_LOADER_SIZE              (2*1024*1024)       /*   2 MB */
#define NAND_FLASH_UBOOT_SIZE               (2*1024*1024)       /*   2 MB */
#define NAND_FLASH_SIGNATURE_SIZE           (2*1024*1024)       /*   2 MB */
#define NAND_FLASH_KERNEL_SIZE              (3*1024*1024)       /*   3 MB */
#define NAND_FLASH_PERM_SIZE                (8*1024*1024)       /*   8 MB */
#define NAND_FLASH_SINGLE_CHANNELLIST_SIZE  (512*1024)          /* 512 KB to support 5000 channels */
#define NAND_FLASH_PQ_SIZE                  (5*1024*1024)       /*   5 MB */
#define NAND_FLASH_AQ_SIZE                  (256*1024)          /* 256 KB */
#define NAND_FLASH_PANEL_SIZE               (256*1024)          /* 256 KB */
#define NAND_FLASH_CIPLUS_SIZE              (256*1024)          /* 256 KB */
#define NAND_FLASH_EDID_SIZE                (256*1024)          /* 256 KB */
#define NAND_FLASH_SERVICE_MODE_SIZE        (256*1024)          /* 256 KB */
#define NAND_FLASH_DVBS_DEFAULT_DB_SIZE     (256*1024)          /* 256 KB */
#define NAND_FLASH_EPG_CACHE_SIZE           (256*1024)          /* 256 KB (Resv) */
#define NAND_FLASH_HDMI_SIZE                (256*1024)          /* 256 KB */
#define NAND_FLASH_WFDP_SIZE                (256*1024)          /* 256 KB */
#define NAND_PART_MAX_IMG_SIZE              (40*1024*1024)      /*  40 MB */
#ifdef REPLACE_EEPROM_WITH_FLASH
#ifdef REPLACE_EEPROM_WITH_FLASH_BLOCK_SIZE_AUTO_DETECTION
#define NAND_FLASH_REPLACE_EEPROM_SIZE      (10*1024*1024)      /*  10 MB */
#else
#define NAND_FLASH_REPLACE_EEPROM_SIZE      (256*1024)          /* 256 KB (but reserved SDM 20MB for virtual eeprom) */
#endif
#endif
#define NAND_FLASH_PQ_EEP_BACKUP_SIZE             (256*1024)          /* 256 KB */
#define NAND_FLASH_RESERVE_SIZE             (256*1024)          /* 256 KB */

#define NAND_PART_SIZE_0                    (NAND_FLASH_LOADER_SIZE)                /* loader image                  */
#define NAND_PART_SIZE_1                    (NAND_FLASH_UBOOT_SIZE)                 /* nand_1  - uboot env           */
#define NAND_PART_SIZE_2                    (NAND_FLASH_SIGNATURE_SIZE)             /* nand_2  - sig A               */
#define NAND_PART_SIZE_3                    (NAND_FLASH_SIGNATURE_SIZE)             /* nand_3  - sig B               */
#define NAND_PART_SIZE_4                    (NAND_FLASH_KERNEL_SIZE)                /* nand_4  - kernel A            */
#define NAND_PART_SIZE_5                    (NAND_FLASH_KERNEL_SIZE)                /* nand_5  - kernel B            */
#define NAND_PART_SIZE_6                    (NAND_FLASH_ROOTFS_SIZE)                /* nand_6  - rootfs A            */
#define NAND_PART_SIZE_7                    (NAND_FLASH_ROOTFS_SIZE)                /* nand_7  - rootfs B            */
#define NAND_PART_SIZE_8                    (NAND_FLASH_APPIMAGE_SIZE)              /* nand_8  - basic               */
#define NAND_PART_SIZE_9                    (NAND_FLASH_3RD_RW_SIZE)                /* nand_9  - 3rd_rw              */
#define NAND_PART_SIZE_10                   (NAND_FLASH_PERM_SIZE)                  /* nand_10 - perm                */
#define NAND_PART_SIZE_11                   (NAND_FLASH_RESERVE_SIZE)               /* nand_11 - reserved            */
#define NAND_PART_SIZE_12                   (NAND_FLASH_3RD_SIZE)                   /* nand_12 - 3rd_ro              */
#define NAND_PART_SIZE_13                   (NAND_FLASH_SINGLE_CHANNELLIST_SIZE)    /* nand_13 - channel list A      */
#define NAND_PART_SIZE_14                   (NAND_FLASH_SINGLE_CHANNELLIST_SIZE)    /* nand_14 - channel list B      */
#define NAND_PART_SIZE_15                   (NAND_FLASH_PQ_SIZE)                    /* nand_15 - Flash PQ            */
#define NAND_PART_SIZE_16                   (NAND_FLASH_AQ_SIZE)                    /* nand_16 - Flash AQ            */
#define NAND_PART_SIZE_17                   (NAND_FLASH_PANEL_SIZE)                 /* nand_17 - Flash Panel         */
#define NAND_PART_SIZE_18                   (NAND_FLASH_CIPLUS_SIZE)                /* nand_18 - ci                  */
#define NAND_PART_SIZE_19                   (NAND_FLASH_EDID_SIZE)                  /* nand_19 - edid                */
#define NAND_PART_SIZE_20                   (NAND_FLASH_SERVICE_MODE_SIZE)          /* nand_20 - Service mode        */
#define NAND_PART_SIZE_21                   (NAND_FLASH_DVBS_DEFAULT_DB_SIZE)       /* nand_21 - DVBS Default DB     */
#define NAND_PART_SIZE_22                   (NAND_FLASH_EPG_CACHE_SIZE)             /* nand_22 - EPG Cache           */
#define NAND_PART_SIZE_23                   (NAND_FLASH_HDMI_SIZE)                  /* nand_23 - HDMI HDCP           */
#define NAND_PART_SIZE_24                   (NAND_FLASH_WFDP_SIZE)                  /* nand_24 - Miracast HDCP       */
#ifdef REPLACE_EEPROM_WITH_FLASH
#define NAND_PART_ID_EEPROM_A               (25)
#define NAND_PART_ID_EEPROM_B               (26)
/* Note:
        1. The virtual eeprom partition size must be equal to nand block size (ex: 128KB, 256KB). 
        2. We reserved 20MB as SDM guarantee partition size for virtual eeprom
*/
#define NAND_PART_SIZE_25                   (NAND_FLASH_REPLACE_EEPROM_SIZE)        /* nand_25 - EEPROM A            */
#define NAND_PART_SIZE_26                   (NAND_FLASH_REPLACE_EEPROM_SIZE)        /* nand_26 - EEPROM B            */
#define NAND_PART_ID_PQ_EEP_BACKUP              (27)
#define NAND_PART_SIZE_27                       (NAND_FLASH_PQ_EEP_BACKUP_SIZE) 
#else
#define NAND_PART_ID_PQ_EEP_BACKUP              (25)
#define NAND_PART_SIZE_25                       (NAND_FLASH_PQ_EEP_BACKUP_SIZE) 
#endif



#define NAND_PART_ENCRYPT_6                (1)                 /* rootfs */
#define NAND_PART_ENCRYPT_7                (1)                 /* rootfs */
#define NAND_PART_ENCRYPT_9                (1)                 /* 3rd_rw */
#define NAND_PART_ENCRYPT_10               (1)                 /* perm */
#define NAND_PART_ENCRYPT_11               (1)                 /* reserved */

/* The NAND_PART_NAME_0-7 use default names. */
//#define NAND_PART_NAME_8                    "basicA"
#define NAND_PART_NAME_9                    "3rd_rw"
#define NAND_PART_NAME_10                   "perm"
#define NAND_PART_NAME_11                   "reserved"
#define NAND_PART_NAME_12                   "3rd_ro"
#define NAND_PART_NAME_13                   "channelA"
#define NAND_PART_NAME_14                   "channelB"
#define NAND_PART_NAME_15                   "pq"
#define NAND_PART_NAME_16                   "aq"
#define NAND_PART_NAME_17                   "panel"
#define NAND_PART_NAME_18                   "ci"
#define NAND_PART_NAME_19                   "edid"
#define NAND_PART_NAME_20                   "svc"
#define NAND_PART_NAME_21                   "ddb"
#define NAND_PART_NAME_22                   "epg"
#define NAND_PART_NAME_23                   "hdmi"
#define NAND_PART_NAME_24                   "wfdp"
#define NAND_PART_NAME_25                   "eepromA"
#define NAND_PART_NAME_26                   "eepromB"

#define NAND_FLASH_STATIC_LAST_PART_ID      (12)

#ifndef REPLACE_EEPROM_WITH_FLASH
#define NAND_PART_END                       (25)                /* total 24 Partitions */
#else
#define NAND_PART_END                       (27)                /* total 26 Partitions with REPLACE_EEPROM_WITH_FLASH */
#endif

#define NAND_FLASH_STATIC_PART_SIZE \
        ((NAND_PART_SIZE_0) + \
        (NAND_PART_SIZE_1) + (NAND_PART_SIZE_2) + (NAND_PART_SIZE_3) + \
        (NAND_PART_SIZE_4) + (NAND_PART_SIZE_5) + (NAND_PART_SIZE_6) + \
        (NAND_PART_SIZE_7) + (NAND_PART_SIZE_8) + (NAND_PART_SIZE_9) + \
        (NAND_PART_SIZE_10) + (NAND_PART_SIZE_11) + (NAND_PART_SIZE_12))

/*Flash PQ customization*/
#ifdef SUPPORT_FLASH_PQ
#define FLASH_PQ_USE_NAND_FLASH                 (1)
#define FLASH_PQ_BASE_PARTITION                 (15)

#ifndef DEMO_BOARD
/* customer */
#define SUPPORT_FLASH_PQ_ENABLE_QTY_SMART_PIC   (1)
#define SUPPORT_FLASH_PQ_ENABLE_GAMMA           (0)
#define SUPPORT_FLASH_PQ_ENABLE_QTY_MAX_MIN     (1)
#define SUPPORT_NVM_GAMMA                       (1)
#endif
#endif

#ifdef SUPPORT_FLASH_AQ
#define FLASH_AQ_BASE_PARTITION             (16)
#define FLASH_AQ_BASE_ADDRESS               (0)
#endif

/* Flash Panel Table */
#ifdef CC_PANEL_FROM_FLASH
#ifdef CC_SOEM_BOARD
#define PANEL_FLASH_HEADER_ADDRESS          (0)
#define PANEL_FLASH_HEADER_LENGTH           (32)
#define PANEL_FLASH_TABLE_SIZE              (176)
#define PANEL_FLASH_TABLE_BASE_PARTITION    (17)
#define PANEL_FLASH_TABLE_ADDRESS           (32)
#endif
#endif

/* Flash EDID */
#define HDMIEDID_NAND_FLASH_PART_ID         (19)
#define HDMIEDID_NAND_FLASH_OFFSET          (0)
#define HDMIEDID_NAND_FLASH_PART_SIZE       (NAND_FLASH_EDID_SIZE)
#define VGAEDID_NAND_FLASH_PART_ID          (19)
#define VGAEDID_NAND_FLASH_OFFSET           (0)
#define VGAEDID_NAND_FLASH_PART_SIZE        (NAND_FLASH_EDID_SIZE)

/* Flash HDCP */
#define HDCP_NAND_FLASH_PART_ID             (23)
#define HDCP_NAND_FLASH_OFFSET              (0)

#define HDCP2X_NAND_FLASH_PART_ID			(24)

#endif /* FLASH_LAYOUT_H */
