/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
#include "general.h"
#include "hw_vdoin.h"
#include "util.h"
#include "pe_if.h"
#include "drvcust_if.h"
#include "mute_if.h"
#include "CustomCtrl.h"
#include "drv_meter.h"
#include "hw_di_int.h"
#include "hw_tcon.h"
#include "x_pinmux.h"
#include "drv_pwm.h"
#include "drv_display.h"
#include "osd_drvif.h"
#include "SelfDiag.h"
#include "x_gpio.h"

// === EXTERN FUNCTION ====================================================================
extern UINT8 vDrvGetLCDFreq(void) ; 
//====================================================================================
void vDRVCUST_BLSet_IP_REF(UINT32 u4Dimming);
void DRVCUST_SetLEDMotion(UINT8 u1OnOff);
UINT16 DRVCUST_EEIModify(UINT16 u2Duty);


// === DEFINE =============================================================================
#define AbsDiff(a, b)  (((a)>(b)) ? ((a)-(b)) : ((b)-(a)))
#define DBC_CLIP(val, min, max) ((val>=max) ? max : ((val<=min) ? min : val))
#define DBC_HIST_NORMBASE    (1000)
#define GAIN_BASE            (1024)

#define IBIZA_DBC_MAGIC00 0x30

// === Static VARIABLE ====================================================================
static UINT8 _ui1_pwm = 255;
static UINT8 u1Coef_ResetCount = 0;
static UINT8 u1Is50HZ=SV_TRUE, u1preIs50HZ = 0xFF;


// === GLOBAL VARIABLE ====================================================================
typedef struct _IB5363_FORCE_BL
{
    UINT8 bDBC_MAGIC;
    UINT8 bBrightLevel;    
} _IB5363_FORCE_BL;

typedef enum
{
    E_DBC_APL,
    E_DBC_BK,
    E_DBC_MAX_DET,
    E_DBC_GFX,
    E_DBC_MAX,
} E_DBC_PARAM_TYPE;

typedef struct
{
    UINT8 u1In;
    UINT8 u1AbsDiff;
    UINT8 u1Coef;
    UINT8 u1Result;
    UINT8 u1IIR_TH[4];    
    UINT8 u1IIR_WEI[3];
} S_DBC_PARAM_IIR;

typedef struct
{
    UINT8 u1Enable;
    UINT8 u1TH[2];    // DBC_XXX_TH0, DBC_XXX_TH1
    UINT8 u1WEI[2];
    UINT8 u1Gain_Result;
} S_DBC_PARAM_CALC;

typedef struct _DBC_STATUS
{
    // --Enable Control--
    UINT8 u1BL_UPDATE_EN;
    UINT8 u1DBC_EN;
    UINT8 u1DBC_PS_EN;
    UINT8 u1DBC_CRI_EN;
    UINT8 u1ZEROD_EN;
    UINT8 u1BLINK_EN;
    
    // --IN/OUT Status--
    UINT8 u1UI_BL_Level;
    UINT8 u1UI_PS_Mode;
    UINT8 u1IP_REF_Dimming;
    UINT8 u1PWM_Dimming;
    UINT8 u1PWM_Dimming_Final;
    
    // ---MISC---
    UINT8 u1LUMA_CORRECT;
    UINT16 u2CURRENT_MIN;
    UINT16 u2CURRENT_MAX;
    UINT16 u2PWM_RATIO_BLIM;
    UINT16 u2PWM_FRQ;
    
    // --SBC --
    UINT8  u1SBC_GAIN;
    UINT8  u1SBC_BL_PSH;
    UINT8  u1SBC_BL_TOP;
    UINT8  u1SBC_BL_MIDDLE;
    UINT8  u1SBC_BL_BOTTOM;
    UINT8  u1SBC_BL_SETUDEN;
	UINT8  u1SBC_BL_PSH_CARBAT;
	UINT8  u1SBC_BL_PSH_PWRBANK;
	UINT8  u1SBC_BL_PSH_NORM;

    // --DBC --
    UINT8  u1DBC_GAIN;

    // --Power Saving /CRI--

    UINT8  u1DBC_PS_LUMA_TH_DIG;
    UINT8  u1DBC_PS_LUMA_TH_ANA;
    UINT8  u1DBC_CRI_LUMA_TH;
	UINT8  u1DBC_PS_SAT_TH;
    UINT16 u2DBC_PS_TGT;
    UINT8  u1DBC_PS_TM_ENTER;
    UINT8  u1DBC_PS_TM_EXIT;
    UINT8  u1DBC_PS_ATN_DOWN;
    UINT8  u1DBC_PS_ATN_UP;
    UINT8  u1DBC_PS_OPERATE;
    UINT8  u1DBC_PS_PIC_STATE;
    UINT8  u1DBC_PS_IN_STATE;

    UINT8  u1HEAT_BLSAVE;
    UINT8  u1HEAT_BLSAVE_GAIN;

    UINT8  u1BLINK_GAIN;
    UINT8  u1BLINK_GAIN_TGT;
    UINT8  u1BLINK_PHASE;
    UINT8  u1BLINK_DUTY;
} DBC_STATUS;

_IB5363_FORCE_BL DBC_debug;
S_DBC_PARAM_IIR DbcParam_IIR[E_DBC_MAX];
S_DBC_PARAM_CALC DbcParam_CALC[E_DBC_MAX];
DBC_STATUS  DBC_Reg; //Dynamic Backlight Control

AUTO_BACKLIGHT_SETS_INFO_T DBC_UI;
UINT8 bLightSensor=0xFF;
UINT8 bEPGFlag = 0xFF;
UINT8 bUSBPhotoPlayback;
UINT8 bFMRadioPlayback;
UINT8 bEnterSYNCMENU;
UINT8 bPICBACKLTONOFF=0xFF;
UINT8 bTeleText_ONOFF = 0xFF;
UINT8 bMHEG_ONOFF = 0xFF;
UINT8 bGinga_ONOFF = 0xFF;

static BOOL fgPWOn_LogoFinish = FALSE;
BOOL fg_OSD_Exist = TRUE;
BOOL fg_PS_TriggerMODE = 0;
BOOL fg_PS_Backlight_On = TRUE;

UINT16 u2PS_InCount=0,u2PS_OutCount=0;
UINT16 wPreDimmer_out;


//--LED Motion--
extern BOOL u1LEDMotionFlag;
extern UINT16 u2LEDMotionDuty;
UINT16 u4ScanPWMPhase50Hz=0;
UINT16 u4ScanPWMPhase60Hz=0;

// for PCC spec use    
//static UINT16 u2BL_POWER_REQ = 0xFF;
// for BL_MANU api
static BOOL fgPWMFixed = FALSE;


//--Power Sequence--
extern BOOL _bfgCustPowerSeqOnOff;
extern BOOL _bfgCustPowerOnSeqPWMON;

//for ZeroD flicker on IPTV
extern UINT16 u2OpcUnMuteCount;

BOOL _bDemoMode = FALSE;
BOOL _bTorchLightOnOff = FALSE;

//////////////////////////////////////////
void vDRCUST_SetPowerOnLogoStatusFlag(UINT8 bflag)
{
    //UNUSED(bflag);
    fgPWOn_LogoFinish = (BOOL)bflag;
}

/**************************************************************************/
//         D_INTERFACE Adaptive Backlight Param Update
/**************************************************************************/
void DRVCUST_AutoBacklightUpdate(AUTO_BACKLIGHT_SETS_INFO_T *aBLight)
{
    //UNUSED(aBLight);
    //UINT32 dwLightsensor = IO32ReadFldAlign(DBC_0E, LightSensorValue);
    static UINT8 bcount = 0;
    bcount = (bcount<255) ? (bcount+1) : 255;
    DBC_UI.ui1BacklightLevel = aBLight->ui1BacklightLevel;
    DBC_UI.ui1PowerSavingMode = aBLight->ui1PowerSavingMode;    
    DBC_UI.ui1LightSensorOnOff = aBLight->ui1LightSensorOnOff;
	bLightSensor = (UINT8)(DBC_UI.ui1LightSensorOnOff);
    
    vIO32WriteFldAlign(BL_CONTROL_MAIN_01, (UINT8)(aBLight->ui1BacklightLevel), UI_BL_LEVEL);
    vIO32WriteFldAlign(BL_CONTROL_MAIN_01, (UINT8)(aBLight->ui1PowerSavingMode), UI_BL_PS_MODE);
	printf("=>BL Level[%d], Power Saving[%d], Light sensor[%d]\n", DBC_UI.ui1BacklightLevel, DBC_UI.ui1PowerSavingMode, DBC_UI.ui1LightSensorOnOff);
    return;
}
extern UINT8 _u1Power_mode;
void DRVCUST_TorchLightCMD( UINT8 ptype,UINT8 psave,UINT8 Tonoff)
{
	_u1Power_mode=ptype;
	
	vIO32WriteFldAlign(BL_CONTROL_MAIN_01, (UINT8)psave, UI_BL_PS_MODE);
	_bTorchLightOnOff=Tonoff;
	vIO32WriteFldAlign(BL_CONTROL_MAIN_00, Tonoff, BL_UPDATE_EN);
	printf("type=%d,save=%d,tonoff=%d\n",ptype,psave,Tonoff);
}

void DRVCUST_DBC_DEBUG_MODE(UINT8 bDBC_MAGIC, UINT8 bBrightLevel)
{
    DBC_debug.bDBC_MAGIC = bDBC_MAGIC;
    DBC_debug.bBrightLevel = bBrightLevel;
        
    if(bDBC_MAGIC==IBIZA_DBC_MAGIC00)
    {
        vIO32WriteFldAlign(BL_CONTROL_MAIN_00, SV_FALSE, BL_UPDATE_EN);
        fgPWMFixed = TRUE;
        //DRVCUST_SetBacklightDuty(VDP_ARG_BACKLT_DBL, DBC_debug.bBrightLevel);
        vDRVCUST_BLSet_IP_REF((DBC_Reg.u2CURRENT_MAX+2)>>2);
        DRVCUST_SetBacklightDuty(VDP_ARG_BACKLT_DBL, 255);
    }
    else
    {
        vIO32WriteFldAlign(BL_CONTROL_MAIN_00, SV_TRUE, BL_UPDATE_EN);
        fgPWMFixed = FALSE;
    }

}

UINT8 DRVCUST_ReadDynamicBacklight(void)
{
    //return 0 if dynamic backlight is off
    return (UINT8)IO32ReadFldAlign(BL_CONTROL_MAIN_00, BL_UPDATE_EN);
}
void DRVCUST_WriteDynamicBacklight(UINT8 ui1_onOff)
{
    if(ui1_onOff)
    {
        _ui1_pwm = 255;
        DRVCUST_DBC_DEBUG_MODE(0x00, _ui1_pwm);
    }
    else
    {
        _ui1_pwm = 255;
        DRVCUST_DBC_DEBUG_MODE(IBIZA_DBC_MAGIC00, _ui1_pwm);
    }
}
UINT8 DRVCUST_ReadPMX(void)
{
    return _ui1_pwm;
}
void DRVCUST_WritePMX(UINT8 ui1_pwm_in)
{
    _ui1_pwm = ui1_pwm_in;
    
    DBC_Reg.u2CURRENT_MAX = IO32ReadFldAlign(BL_CONTROL_MAIN_03, CURRENT_MAX);//temp
    //vDRVCUST_BLSet_IP_REF((DBC_Reg.u2CURRENT_MAX+2)>>2);
    vDRVCUST_BLSet_IP_REF(DBC_Reg.u1IP_REF_Dimming);
    DRVCUST_SetBacklightDuty(VDP_ARG_BACKLT_DBL, MIN(ui1_pwm_in,255));
}

BOOL DRVCUST_GetLEDMotionFlag(void)
{
	return u1LEDMotionFlag;
}

void DRVCUST_EnableLEDMotionDebugMode(UINT8 u1OnOff)
{
	//vIO32WriteFldAlign(DBC_00, u1OnOff, DBC_SCANPWM_Debug_ENA);	
}

void DRVCUST_USBPHOTO_MODE(UINT8 bMode)
{
    switch(bMode)
    {
//           case D_MMP_NONE: //Normal mode
            //bUSBPhotoPlayback = SV_FALSE;
            //Printf("D_MMP_NONE XXX \n");
            //break;
        case D_MMP_PHOTO: //Turn on adaptive backlight on the USB Photo playback mode
    		bUSBPhotoPlayback = SV_TRUE;
            //Printf("D_MMP_PHOTO XXX \n");
    		break;
//           case D_MMP_MUSIC: //Normal mode
            //bUSBPhotoPlayback = SV_FALSE;
            //Printf("D_MMP_MUSIC XXX \n");
            //break;
//          case D_MMP_MOVIE: //Normal mode
            //bUSBPhotoPlayback = SV_FALSE;
            //Printf("D_MMP_MOVIE XXX \n");
            //break;
  		default:
    		bUSBPhotoPlayback = SV_FALSE;
    		break;
   }
}	

void DRVCUST_USBOSD_MODE(UINT8 bMode)
{
    switch(bMode)
    {
//           case D_MMP_OSD_NONE: //Normal mode
            //bUSBPhotoPlayback = SV_FALSE;
            //Printf("D_MMP_OSD_NONE XXX \n");
            //break;
        case D_MMP_OSD_THUMBNAIL: //Normal mode
    		bUSBPhotoPlayback = SV_FALSE;
            //Printf("D_MMP_OSD_THUMBNAIL XXX \n");
    		break;
//            case D_MMP_OSD_DEVICE: //Normal mode
            //bUSBPhotoPlayback = SV_FALSE;
            //Printf("D_MMP_OSD_DEVICE XXX \n");
            //break;
//            case D_MMP_OSD_PLAYBACK: //Normal mode
            //bUSBPhotoPlayback = SV_FALSE;
            //Printf("D_MMP_OSD_PLAYBACK XXX \n");
            //break;
   }
}	

void DRVCUST_EPG_FLAG(UINT8 bMode)
{
    switch(bMode)
    {
        case D_MMP_OSD_NOT_FULL: //Normal mode
    		bEPGFlag = SV_FALSE;
            LOG(2, "DDD== D_EPG_FALSE XXX \n");
    		break;
        case D_MMP_OSD_FULL: //Turn on adaptive backlight on the USB Photo playback mode
    		bEPGFlag = SV_TRUE;
            LOG(2, "DDD== D_EPG_TRUE XXX \n");
    		break;
//    bEPGFlag= bOnOff? SV_TRUE : SV_FALSE;
    }
}	

void DRVCUST_FMRADIO_FLAG(UINT8 bMode)
{
    switch(bMode)
    {
        case D_FM_RADIO_OFF: //Normal mode
    		bFMRadioPlayback = SV_FALSE;
            //Printf("D_FM_RADIO_OFF XXX \n");
    		break;
        case D_FM_RADIO_ON: //Turn on adaptive backlight on the USB Photo playback mode
    		bFMRadioPlayback = SV_TRUE;
            //Printf("D_FM_RADIO_ON XXX \n");
    		break;
		default:
			bFMRadioPlayback = SV_FALSE;
    		break;			
//    bEPGFlag= bOnOff? SV_TRUE : SV_FALSE;
    }
}	

void DRVCUST_SYNC_MENU_FLAG(UINT8 bMode)
{
    switch(bMode)
    {
        case D_SYNC_MENU_OFF: //Normal mode
    		bEnterSYNCMENU = SV_FALSE;
    		Printf("D_SYNC_MENU_OFF XXX \n");
    		break;
        case D_SYNC_MENU_ON: //Fix backlight when enter SYNC_MENU
    		bEnterSYNCMENU = SV_TRUE;
    		Printf("D_SYNC_MENU_ON XXX \n");
    		break;
		default:
			bEnterSYNCMENU = SV_FALSE;
	        break;			
    }
}

// add for picture off on backlight sequence
void DRVCUST_PICBACKLT_FLAG(UINT8 bMode)
{
    switch(bMode)
    {
        case D_PICBACKLT_OFF: // pictrue off with backlight off
    		bPICBACKLTONOFF = SV_FALSE;
            vDRVCUST_BLSet_IP_REF(0);
            DRVCUST_SetBacklightDuty(VDP_ARG_BACKLT_DBL, (UINT8)0x0);
            DRVCUST_SetScanPWM_High_Low(0);
    		Printf("D_PICBACKLT_OFF XXX \n");
    		break;
        case D_PICBACKLT_ON: //Turn on  backlight
    		bPICBACKLTONOFF = SV_TRUE;
            vDRVCUST_BLSet_IP_REF(DBC_Reg.u1IP_REF_Dimming);
            DRVCUST_SetBacklightDuty(VDP_ARG_BACKLT_DBL, DBC_Reg.u1PWM_Dimming_Final);
            DRVCUST_SetLEDMotion(DRVCUST_GetLEDMotionFlag());
    		Printf("D_PICBACKLT_ON XXX \n");
    		break;
        default:
            break;
    }
}	

UINT8 DRVCUST_GetPICBACKLT_FLAG(void)
{
	return bPICBACKLTONOFF;
}

void DRVCUST_SetScanPWMPhase(UINT32 u4Phase)
{
//	vIO32WriteFldAlign(DBC_00, u4Phase, DBC_SCANPWM_PHASE);		
	if ((vDrvGetLCDFreq() == 25) || (vDrvGetLCDFreq() == 50) || (vDrvGetLCDFreq() == 100))	// 50Hz
	{	
		u4ScanPWMPhase50Hz = u4Phase;		
		LOG(4, "DRVCUST_SetScanPWMPhase() u4ScanPWMPhase50Hz = %d\n", u4ScanPWMPhase50Hz); 
	}
	else		// 60Hz
	{
		u4ScanPWMPhase60Hz = u4Phase;		
		LOG(4, "DRVCUST_SetScanPWMPhase() u4ScanPWMPhase60Hz = %d\n", u4ScanPWMPhase60Hz);   		
	}
}

void DRVCUST_TeleText_FLAG(UINT8 bMode)
{
    switch(bMode)
    {
        case D_TELETEXT_ON: //TeleText exist
    		bTeleText_ONOFF = SV_ON;
            LOG(2, "DDD== D_TeleText_ON XXX \n");
    		break;
       case D_TELETEXT_OFF: //TeleText not exist
           bTeleText_ONOFF = SV_OFF;
           LOG(2, "DDD== D_TeleText_OFF XXX \n");
           break;
  		default:
    		break;
   }
}	

void DRVCUST_MHEG_FLAG(UINT8 bMode)
{
    switch(bMode)
    {
        case D_MHEG_ON: //MHEG exist
    		bMHEG_ONOFF = SV_ON;
            LOG(2, "DDD== D_MHEG_ON XXX \n");
    		break;
       case D_MHEG_OFF: //MHEG not exist
           bMHEG_ONOFF = SV_OFF;
           LOG(2, "DDD== D_MHEG_OFF XXX \n");
           break;
  		default:
    		break;
   }
}	

void DRVCUST_GINGA_FLAG(UINT8 bMode)
{
    switch(bMode)
    {
        case D_GINGA_ON: //Ginga exist
    		bGinga_ONOFF = SV_ON;
            LOG(2, "DDD== D_Ginga_ON XXX \n");
    		break;
       case D_GINGA_OFF: //Ginga not exist
           bGinga_ONOFF = SV_OFF;
           LOG(2, "DDD== D_Ginga_OFF XXX \n");
           break;
  		default:
    		break;
   }
}	

//-----------------------------------------------------------------------------
/** set panel IP_REF dimming.
 *  @param u4Dimming: Brightness level.
 *                        PWM(0:dark --> 255:bright)
 *                        Scan PWM(0%:dark --> 100%:bright)
 *  @return N/A
 */
//-----------------------------------------------------------------------------
void vDRVCUST_BLSet_IP_REF(UINT32 u4Dimming)
{
    #define IP_REF_PwmPort (SrcPWM1)
    #define PWM_DUTY_ALL 255
    #define PWM_DUTY_LESS 254
    
    UINT32 u4Frequency = IO32ReadFldAlign(BL_CONTROL_MAIN_02, BL_IPREF_FRQ);//PANEL_Get_DBC_CurrentRatio_Freq();
    
    if(u4Frequency == 0)
    {
        u4Frequency = 80000;
    }

    u4Dimming = (u4Dimming >= 256) ? 256 : u4Dimming;
    
#ifdef CC_SOEM_BOARD
    if((u4Dimming == 256)||(u4Dimming == 255))
    {
        vDrvSetPWM(IP_REF_PwmPort,
                    (((BSP_GetDomainClock(SRC_BUS_CLK)) / (PWM_DUTY_LESS+1))
                        + (u4Frequency>>1))/ u4Frequency,
                    0xFF, PWM_DUTY_LESS);
    }
    else
    {
        vDrvSetPWM(IP_REF_PwmPort,
                   ((BSP_GetDomainClock(SRC_BUS_CLK) >> 8)+(u4Frequency>>1)) / u4Frequency, 
                    u4Dimming, PWM_DUTY_ALL);
    }
#endif

    LOG(3, ">>> vDRVCUST_BLSet_IP_REF:Dimming[%d],Freq[%d]\n", u4Dimming,u4Frequency);
}


/**************************************************************************/
// for picture off on mode backligt sequence control, should be modified for difference IC.
/**************************************************************************/
void DRVCUST_SetScanPWM_High_Low(UINT32 u4High)
{
	vDrvSetScanPWM(SrcPWM2, 0, u4High, 0);
    vDrvSetScanPWMOnOff(SrcPWM2, SV_ON);
    vIO32WriteFldAlign(PWM_SCAN_06, 0, REG_PWM_SCAN2_POL);
    vDrvSetScanPWMLatchMode(1);
    vDrvSetScanPWMLatchMode(0);
    BSP_PinSet(PIN_OPWM3, PINMUX_FUNCTION3); // dimmer 2  (LED motion setting)
}

/**************************************************************************/
//        LED Motion - ScanPWM
/**************************************************************************/
void DRVCUST_SetLEDMotion(UINT8 u1OnOff)
{	
    UINT8 u1Phase = DBC_Reg.u1BLINK_PHASE;  //0~255 -> 0~tFrame
    UINT8 u1BacklightDuty = MIN(DBC_Reg.u1BLINK_DUTY,100);  //0~100
    UINT8 u1FreqMutiple=2;
    
	UINT16 u2DutyRange;
    UINT16 u2ScanPWM_Phase,u2ScanPWM_High,u2ScanPWM_Low;

    //if (fgPWOn_LogoFinish != TRUE)    //for power on case
    //{
    //    return;
    //}

    vIO32WriteFldAlign(BLINK_PARAM, u1OnOff, LEDMOTION_MODE);
    //--For debug
    if(IO32ReadFldAlign(BLINK_DEBUG, BLINK_DEBUG_EN))    
    {
        u1Phase = IO32ReadFldAlign(BLINK_DEBUG, BLINK_DEBUG_PHASE);
        u1FreqMutiple = IO32ReadFldAlign(BLINK_DEBUG, BLINK_DEBUG_Mutiple);
        u1BacklightDuty = IO32ReadFldAlign(BLINK_DEBUG, BLINK_DEBUG_DUTY);
        u1OnOff= IO32ReadFldAlign(BLINK_DEBUG, BLINK_DEBUG_ONOFF);
        Printf("\nDDD--DRVCUST_SetLEDMotion,phase[%d]->[%d],duty[%d]->[%d]--\n",
            DBC_Reg.u1BLINK_PHASE,u1Phase,DBC_Reg.u1BLINK_DUTY,u1BacklightDuty);
    }

    //--LEDMotion Off, Fix high
    if(u1OnOff==SV_OFF)
    {
        if(_bfgCustPowerSeqOnOff)//(bIsPanelOn())//for power sequence spec
        {
            DRVCUST_SetScanPWM_High_Low(1000);
        }
        return ;
    }

    //--LEDMotion On, Set ScanPWM
	u1FreqMutiple = (u1FreqMutiple == 0 ? 1 : u1FreqMutiple);
	u2DutyRange = 1000/u1FreqMutiple;
    
    u2ScanPWM_Phase = (u1Phase==255) ? 0 : (u1Phase*u2DutyRange/255);
    u2ScanPWM_High = u2DutyRange*u1BacklightDuty/100;
	u2ScanPWM_Low =  u2DutyRange - u2ScanPWM_High;	

    vDrvSetScanPWMOnOff(SrcPWM2, SV_ON);
	vDrvSetScanPWMLatchMode(0);  // Aligned Vsync
	vIO32WriteFldAlign(PWM_SCAN_12, 0, REG_PWM_SCAN1_START_MODE);

	if(u2ScanPWM_Phase <= u2ScanPWM_Low)			
	{	
	    u2ScanPWM_Low = u2ScanPWM_Low - u2ScanPWM_Phase;
		vIO32WriteFldAlign(PWM_SCAN_06, 0, REG_PWM_SCAN2_POL);
	}
	else
	{				
		u2ScanPWM_Phase = u2ScanPWM_Phase - u2ScanPWM_Low;
		u2ScanPWM_High = u2ScanPWM_Low;
		u2ScanPWM_Low = u2DutyRange - u2ScanPWM_Phase - u2ScanPWM_High;
		vIO32WriteFldAlign(PWM_SCAN_06, 1, REG_PWM_SCAN2_POL);
	}
	vDrvSetScanPWM(SrcPWM2, u2ScanPWM_Phase, u2ScanPWM_High, u2ScanPWM_Low); 	
    vDrvSetScanPWMLatchMode(1);
    vDrvSetScanPWMLatchMode(0);
    if(_bfgCustPowerSeqOnOff)//(bIsPanelOn())//for power sequence spec
    {
    	BSP_PinSet(PIN_OPWM3, PINMUX_FUNCTION3); // dimmer 2  (LED motion setting)
    }
}

void DRVCUST_BacklightProcResume(void)
{
	//Printf("\n****DRVCUST_BacklightProcResume*******\n");
	//SCANPWM_run = SV_TRUE;
    u1Is50HZ=SV_TRUE;
    u1preIs50HZ = 0xFF;
}

//============================================================
//==================  End   =============================


/*************************************************************************************/
//            Customized Adaptive Backlight Driver (External Interfaces)
/*************************************************************************************/
#define EEI_MAGIC1 0x3FF8
#define EEI_MAGIC2 0x3FF9
#define EEIBASE_RF 0x3FFA
#define EEIBASE_YP 0x3FFB
#define EEI_DECREASE_NORMAL 0x3FFC
#define EEI_INCREASE_NORMAL 0x3FFD


/*************************************************************************************/
//            Customized Adaptive Backlight Driver (Internal Interfaces)
/*************************************************************************************/
void DRVCUST_AdaptiveBacklightParam(UINT8 bMode)
{
    // ------------------------------------------------------------------------------------------------------------
    // this function defines the behavior when UI parameter changed
    // for example : UI have 3 level (low (0) / mid (1) / high (2))
    // when level changed, backlight mapping changed according to pre-defined behavior
    // ------------------------------------------------------------------------------------------------------------
	UNUSED(bMode);

}

UINT16 DRVCUST_EEIModify(UINT16 u2Duty)
{
    EXTERN UCHAR VDP_GetPlayMode(UCHAR ucVdpId);
    static UINT8 bEEIAdd=0;
    static UINT8 bCounter=0;
    UINT8 bEEIIncrease = IO32ReadFldAlign(EEI_PARAM_00, DBC_EEI_INCREASE);    
    UINT8 bStep=IO32ReadFldAlign(EEI_PARAM_00, DBC_EEI_STEP);
    UINT8 bSaturation;
	//UINT8 bSourceTimingOK = 0;
    UINT16 waLumaHist[LUMA_HIST_NUM];
    UINT32 wDark, wWhite;
    UINT32 wMotion;
    UINT16 wWhiteTH, wDarkTh;	
	UINT8 bSmartPic;
	UINT16 wPower_saving_sta;
	UINT8 bSignalType = bGetSignalType(SV_VP_MAIN);
	UINT8 bIsUSB = VDP_GetPlayMode(SV_VP_MAIN);
	
	#if 0//def DBC_EEI_CHD
	UINT8 bType = bGetSignalType(SV_VP_MAIN);
	#endif
	
    wDarkTh = IO32ReadFldAlign(EEI_PARAM_01,DBC_EEI_DARK_TH);
    wWhiteTH = IO32ReadFldAlign(EEI_PARAM_01,DBC_EEI_WHITE_TH);
    
	/*if((bGetSignalType(SV_VP_MAIN)==(UINT8)SV_ST_TV && bHwTvdMode()==SV_CS_PAL))
	{
		bSourceTimingOK = 1;
		bEEIIncrease=IO32ReadFldAlign(DBC_15, DBC_EEI_INCREASE_RF);
	}
	if(bGetSignalType(SV_VP_MAIN)==(UINT8)SV_ST_YP)
	{
		bSourceTimingOK = 1;
		bEEIIncrease=IO32ReadFldAlign(DBC_15, DBC_EEI_INCREASE_YP);
	}*/  // mark to apply for all source
    
    u1DrvGetLumaHist(waLumaHist, 512);
    wMotion=RegReadFldAlign(MCVP_STATUS_0F, OLD_FRAME_MO_Q_STATUS);
    wDark = waLumaHist[0]+waLumaHist[1]+waLumaHist[2]+waLumaHist[3];
    wWhite= waLumaHist[LUMA_HIST_NUM-1]+waLumaHist[LUMA_HIST_NUM-2]+
            waLumaHist[LUMA_HIST_NUM-3]+waLumaHist[LUMA_HIST_NUM-4];
    bSaturation= u1DrvGetASL();
	vIO32WriteFldAlign(EEI_PARAM_02, wDark, DBC_EEI_DARK_Count);
	vIO32WriteFldAlign(EEI_PARAM_02, wWhite, DBC_EEI_WHITE_Count);
	
	bSmartPic = vApiFlashPqGetSmartPicId();

	wPower_saving_sta = IO32ReadFldAlign(BL_CONTROL_MAIN_01, UI_BL_PS_MODE);
	
    
    if(bStep!=0)
    {
        if(bCounter%bStep==0)
        {
            if( bSaturation<10 &&
               wMotion < 0x3000 &&
               (wDark > wDarkTh && wWhite > wWhiteTH) &&
               (bSmartPic != 3) &&			 //bSmartPic=3 custom mode will turn off EEI Function.
               (wPower_saving_sta != 1 && wPower_saving_sta != 3) &&	//Not in ECO mode.
               //(bSignalType != SV_ST_DVI) &&    //HDMI disable
               (!((bSignalType == SV_ST_MPEG)&&(bIsUSB == 1))))    //USB disable
            {
                bEEIAdd=(bEEIAdd<bEEIIncrease)?(bEEIAdd+1):bEEIIncrease;

                #if 0//def DBC_EEI_CHD
                if(bSmartPic == 2)    //For standard picture mode, increase bright
                {
                	if(bType == SV_ST_MPEG)
                	{
                		vIO32WriteFldAlign(BRIGHT_PLUS_01, 116, BRIGHT_PLUS_G1);
                	}
                	else
                	{
                    	vIO32WriteFldAlign(BRIGHT_PLUS_01, \
                            ((PANEL_GetPanelWidth() <= 1440) ? 124 : 121), BRIGHT_PLUS_G1);
                    }
                }                
                #endif
            }
            else
            {
                bEEIAdd=(bEEIAdd-1>0)?(bEEIAdd-1):0;
                
                #if 0//def DBC_EEI_CHD
                    vIO32WriteFldAlign(BRIGHT_PLUS_01, \
                        wReadQualityTable(QUALITY_MISC_BRIGHTPLUS_G1), BRIGHT_PLUS_G1);
                #endif
            }
        }
    }
    bCounter++;
    //return MIN((u2Duty+bEEIAdd),dwBackltMax);
    return MIN((u2Duty+bEEIAdd),DBC_Reg.u1SBC_BL_TOP);
    //return MIN((u2Duty+bEEIAdd),((DBC_Reg.u2CURRENT_MAX+2)>>2));
}

void DRVCUST_SetBacklightDuty(UINT8 u1RefIndex, UINT16 u2Duty)
{
    // ------------------------------------------------------------------------------------------------------------    
    // Customized backlight interface
    // Could be defined as multiplication of several result :
    // such as (1) adaptive backlight * (2) UI value * (3) light sensor value
    // ------------------------------------------------------------------------------------------------------------    
    static UINT16 u2BacklightSrc[VDP_ARG_BACKLT_MAX] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
    UINT16 u2BacklightDuty;   
/*
	UINT8 bEEIOnOff=IO32ReadFldAlign(EEI_PARAM_00,DBC_EEI_ON);
	
        //Patch for EEI.
        if(bEEIOnOff)
        {
            u2BacklightSrc[u1RefIndex] = DRVCUST_EEIModify(u2Duty);
        }
        else
        {
            u2BacklightSrc[u1RefIndex] = u2Duty;
        }
*/
    
    u2BacklightSrc[u1RefIndex] = u2Duty;
    
    switch(u1RefIndex)
    {
        case VDP_ARG_BACKLT_UI:
        case VDP_ARG_BACKLT_DBL:
			u2BacklightDuty = u2BacklightSrc[u1RefIndex] ;
			if (0)  //(PANEL_BRIGHT_SETTING == PWM_HIGH_PANEL_BRIGHT)
			{
	               u2BacklightDuty=(0xFF-u2BacklightDuty);
			}		
		    vApiSetPanelBright((u2BacklightDuty));			
            break;         
        case VDP_ARG_BACKLT_LIGHT_SNSR:
            // behavior to be defined by customer
            break;
        case VDP_ARG_BACKLT_OPC:
            // behavior to be defined by customer
            break;            
        case VDP_ARG_BACKLT_SYSTEM_SET:
            u2BacklightDuty = u2BacklightSrc[VDP_ARG_BACKLT_SYSTEM_SET];
            vApiSetPanelBright(u2BacklightDuty);
            break;            
            
    }        
}

/**************************************************************************/
//         Customized Backlight Driver For Loader Boot Sequence
/**************************************************************************/
void DRVCUST_UpdateBacklightState(UINT8 u1State, UINT16 u2Duty)
{
    UNUSED(u1State);
    UNUSED(u2Duty);
}

UINT16 u2DRVCUST_DBC_Get_Power_limit_gain(void)
{
    //#define BL_POWER_LIMIT_IIRP (800)
    //#define BL_POWER_LIMIT_IIRM (400)
#ifndef CC_PowerCons_Ctrl	
    return 1024;
#else
    EXTERN UINT8 C_BL_POWER_LIMIT;    //Get from Salvia_video_drv.c
    
    UINT8 u1BL_POWER_LIMIT = C_BL_POWER_LIMIT;
    static UINT32 u4blPowerlimitGainPre = (1<<18); //initial
    UINT16 u2BL_POWER_LIMIT_TGT,u2PowerLimitGainIir,u2BL_POWER_LIMIT_GAIN;
    
    u2BL_POWER_LIMIT_TGT = ((UINT32)u1BL_POWER_LIMIT * GAIN_BASE+128)/255;
    if(u2BL_POWER_LIMIT_TGT>=((u4blPowerlimitGainPre+(1<<7))>>8))
    {
        u2PowerLimitGainIir = IO32ReadFldAlign(DBC_PCC_00, BL_POWER_LIMIT_IIRP);  //50
    }
    else
    {
        u2PowerLimitGainIir = IO32ReadFldAlign(DBC_PCC_00, BL_POWER_LIMIT_IIRM);  //50
    }

    u4blPowerlimitGainPre = ((UINT32)u2PowerLimitGainIir * u2BL_POWER_LIMIT_TGT * 256
               + (GAIN_BASE - u2PowerLimitGainIir) * u4blPowerlimitGainPre + 512)>>10;
    u2BL_POWER_LIMIT_GAIN = (u4blPowerlimitGainPre+(1<<7))>>8;

    return u2BL_POWER_LIMIT_GAIN;
#endif	
}


void DRVCUST_Reset_DBC_IIRCoef(void)
{
	//Only when scene change reset coef_resetCount
    u1Coef_ResetCount = IO32ReadFldAlign(BL_CONTROL_MAIN_04, DBC_IIR_CLR_TIM);
}

static UINT8 _u1Util_math_step_func(UINT8 u1Diff, UINT8 u1TH[4], UINT8 u1IIR[3])
{
    UINT8 u1Coef = 0;
    if(u1Coef_ResetCount != 0)
    {
        return 255;
    }
    
    if(u1Diff <= u1TH[0])         //---[0 ~ TH0]  =>>  IIR_L
    {
        u1Coef = u1IIR[0];    
    }
    else if(u1Diff <= u1TH[1])    //---[TH0 ~ TH1]  =>>  [IIR_L <-> IIR_M]
    {
        u1Coef = u1IIR[0]+((u1Diff-u1TH[0])*(u1IIR[1]-u1IIR[0])/(u1TH[1]-u1TH[0]));
    }
    else if(u1Diff <= u1TH[2])    //---[TH1 ~ TH2]  =>>  IIR_M
    {
        u1Coef = u1IIR[1];
    }
    else if(u1Diff <= u1TH[3])    //---[TH2 ~ TH3]  =>>  [IIR_M <-> IIR_H]
    {
        u1Coef = u1IIR[1]+((u1Diff-u1TH[2])*(u1IIR[2]-u1IIR[1])/(u1TH[3]-u1TH[2]));
    }
    else                          //---[TH3 ~ 255]  =>>  IIR_H           
    {
        u1Coef = u1IIR[2];    
    }

    return u1Coef;      
}

static UINT8 _u1DrvDBC_Util_IIR(UINT8 bIndex, UINT8 u1Coef, UINT8 u1In, UINT8 u1Result)
{
#if 1
    UINT32 u4Out = 0;
    static UINT32 u4PreOut[E_DBC_MAX] = {0xffffff,0xffffff,0xffffff,0xffffff};
    UNUSED(u1Result);
    
    if (u4PreOut[bIndex] == 0xffffff)
    {
        u4Out = u1In * 8 * 256;
    }
    else
    {
        u4Out = ((((UINT32)(u1In))*(u1Coef+1)*(8*256) + 
                    (u4PreOut[bIndex])*(255-u1Coef) + 128) >> 8);
    }
    
    //===Begin for debug===========
    if (IO32ReadFldAlign(BL_CONTROL_MAIN_01, Fld( 1,  14, AC_MSKB1))
        &&(bIndex == IO32ReadFldAlign(BL_CONTROL_MAIN_01, Fld( 3,  11, AC_MSKB1))))
    {
        printf("--Index[%d] >> In:[%d],Coef[%d],Preout[%d], Out:[%d]-[%d]-- \n ",
                  bIndex, u1In, u1Coef, u4PreOut[bIndex],
                  u4Out,((UINT8)((u4Out+(1<<10)) >> 11)));
    }
    //===End for debug===========
    
    u4PreOut[bIndex] = u4Out;
    return (UINT8)((u4Out+(1<<10)) >> 11);
#else
    UINT8 u1Out = 0;
    static UINT8 u1PreOut[E_DBC_MAX] = {0,0,0,0};
    
    u1Out = (UINT8)((((UINT32)(u1In))*(u1Coef+1) + 
                        ((UINT32)(u1Result))*(255-u1Coef) + 128) >> 8);

    //---Check  the output, avoid output always can not round to input 
    if ((u1Out == u1PreOut[bIndex]) && (u1Out != u1In))
    {
        u1Out = (u1Out < u1In) ? (u1Out+1) : ((u1Out > u1In) ? (u1Out-1) : u1Out);
    }
    u1PreOut[bIndex] = u1Out;

    return u1Out;
#endif
}

static UINT8 _u1DrvDBC_Get_MaxBrigtness(UINT8 u1DbcMaxDetTh)
{
    UINT8 bi,u1dbc_max_det = 0xFF;
    UINT16 DBCLumaMaxNormHist[32];
    UINT16 u2dbc_max_thr,u2dbc_max_sum = 0;

    u2dbc_max_thr = u1DbcMaxDetTh*DBC_HIST_NORMBASE/1000;
    u1DrvGetLumaHist(DBCLumaMaxNormHist, DBC_HIST_NORMBASE);
    for(bi = 31; bi >= 0; bi--)
    {
        u2dbc_max_sum += DBCLumaMaxNormHist[bi];
        if(u2dbc_max_sum >=  u2dbc_max_thr)
            break;
    }
    u1dbc_max_det = MIN((bi)<<3, 255);
    //vIO32WriteFldAlign(BL_CONTROL_MAIN_03, u1dbc_max_det, MAX_DET_MAX_CONTENT);   //TBD===========
    return u1dbc_max_det;
}

static UINT8 _u1DrvDBC_GetBlackIndex(void)
{
	UINT8 i,bBlackIndex,id=0;
	UINT32 dwBlackPxlIdx = 0;
	UINT16 wHistCur[32];
	const UINT16 wBlackWeighting[2][16] = 
	{
    	{24, 24, 24 ,24, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},  //Others use
	    {24, 24, 24 ,19,  8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}   //Cinema(Pic id = 4) use
    };
    id = (vApiFlashPqGetSmartPicId()==4) ? 1 : 0;
    u1DrvGetLumaHist(wHistCur, DBC_HIST_NORMBASE);
	for(i = 0; i < 16; i++)
	{
		dwBlackPxlIdx += wHistCur[i]*wBlackWeighting[id][i];
	}
	bBlackIndex = (UINT8)DBC_CLIP((((dwBlackPxlIdx * 255) + (DBC_HIST_NORMBASE*wBlackWeighting[id][0]>>1))/(DBC_HIST_NORMBASE*wBlackWeighting[id][0])), 0, 255);
	vIO32WriteFldAlign(DBC_PARAM1_BK_01, bBlackIndex, DBC_BK_INDEX);
    return bBlackIndex;
}

static UINT8 _u1DrvDBC_Calc_Gain(UINT8 u1In, UINT8 u1TH[2], UINT8 u1Wei[2])
{
    if(u1In <= u1TH[0])
    {
        return u1Wei[0];
    }
    else if(u1In <= u1TH[1])
    {
        return (UINT8)((INT16)u1Wei[0]+(((INT16)(u1In-u1TH[0]))*
            ((INT16)u1Wei[1]-u1Wei[0])/(u1TH[1]-u1TH[0])));
    }
    else
    {
        return u1Wei[1];
    }
}

static void _vDRVCUST_DBC_IIR_Param_Update(void)
{
    UINT32 u4APL_IIR_TH,u4APL_IIR_WEI,u4BK_IIR_TH,u4BK_IIR_WEI;
    UINT32 u4MAX_DET_IIR_TH,u4MAX_DET_IIR_WEI,u4GFX_IIR_TH,u4GFX_IIR_WEI;

    u4APL_IIR_TH = u4IO32Read4B(DBC_PARAM0_APL_00);
    u4APL_IIR_WEI = u4IO32Read4B(DBC_PARAM0_APL_01);

    u4BK_IIR_TH = u4IO32Read4B(DBC_PARAM0_BK_00);
    u4BK_IIR_WEI = u4IO32Read4B(DBC_PARAM0_BK_01);

    u4MAX_DET_IIR_TH = u4IO32Read4B(DBC_PARAM0_MAX_DET_00);
    u4MAX_DET_IIR_WEI = u4IO32Read4B(DBC_PARAM0_MAX_DET_01);

    u4GFX_IIR_TH = u4IO32Read4B(DBC_PARAM0_GFX_00);
    u4GFX_IIR_WEI = u4IO32Read4B(DBC_PARAM0_GFX_01);

    // APL
    DbcParam_IIR[E_DBC_APL].u1IIR_TH[0] = (UINT8)(u4APL_IIR_TH & 0x0FF);
    DbcParam_IIR[E_DBC_APL].u1IIR_TH[1] = (UINT8)(u4APL_IIR_TH>>8 & 0x0FF);
    DbcParam_IIR[E_DBC_APL].u1IIR_TH[2] = (UINT8)(u4APL_IIR_TH>>16 & 0x0FF);
    DbcParam_IIR[E_DBC_APL].u1IIR_TH[3] = (UINT8)(u4APL_IIR_TH>>24 & 0x0FF);
    DbcParam_IIR[E_DBC_APL].u1IIR_WEI[0] = (UINT8)(u4APL_IIR_WEI & 0x0FF);
    DbcParam_IIR[E_DBC_APL].u1IIR_WEI[1] = (UINT8)(u4APL_IIR_WEI>>8 & 0x0FF);
    DbcParam_IIR[E_DBC_APL].u1IIR_WEI[2] = (UINT8)(u4APL_IIR_WEI>>16 & 0x0FF);

    // BK
    DbcParam_IIR[E_DBC_BK].u1IIR_TH[0] = (UINT8)(u4BK_IIR_TH & 0x0FF);
    DbcParam_IIR[E_DBC_BK].u1IIR_TH[1] = (UINT8)(u4BK_IIR_TH>>8 & 0x0FF);
    DbcParam_IIR[E_DBC_BK].u1IIR_TH[2] = (UINT8)(u4BK_IIR_TH>>16 & 0x0FF);
    DbcParam_IIR[E_DBC_BK].u1IIR_TH[3] = (UINT8)(u4BK_IIR_TH>>24 & 0x0FF);
    DbcParam_IIR[E_DBC_BK].u1IIR_WEI[0] = (UINT8)(u4BK_IIR_WEI & 0x0FF);
    DbcParam_IIR[E_DBC_BK].u1IIR_WEI[1] = (UINT8)(u4BK_IIR_WEI>>8 & 0x0FF);
    DbcParam_IIR[E_DBC_BK].u1IIR_WEI[2] = (UINT8)(u4BK_IIR_WEI>>16 & 0x0FF);

    // MAX_DET
    DbcParam_IIR[E_DBC_MAX_DET].u1IIR_TH[0] = (UINT8)(u4MAX_DET_IIR_TH & 0x0FF);
    DbcParam_IIR[E_DBC_MAX_DET].u1IIR_TH[1] = (UINT8)(u4MAX_DET_IIR_TH>>8 & 0x0FF);
    DbcParam_IIR[E_DBC_MAX_DET].u1IIR_TH[2] = (UINT8)(u4MAX_DET_IIR_TH>>16 & 0x0FF);
    DbcParam_IIR[E_DBC_MAX_DET].u1IIR_TH[3] = (UINT8)(u4MAX_DET_IIR_TH>>24 & 0x0FF);
    DbcParam_IIR[E_DBC_MAX_DET].u1IIR_WEI[0] = (UINT8)(u4MAX_DET_IIR_WEI & 0x0FF);
    DbcParam_IIR[E_DBC_MAX_DET].u1IIR_WEI[1] = (UINT8)(u4MAX_DET_IIR_WEI>>8 & 0x0FF);
    DbcParam_IIR[E_DBC_MAX_DET].u1IIR_WEI[2] = (UINT8)(u4MAX_DET_IIR_WEI>>16 & 0x0FF);

    // GFX
    DbcParam_IIR[E_DBC_GFX].u1IIR_TH[0] = (UINT8)(u4GFX_IIR_TH & 0x0FF);
    DbcParam_IIR[E_DBC_GFX].u1IIR_TH[1] = (UINT8)(u4GFX_IIR_TH>>8 & 0x0FF);
    DbcParam_IIR[E_DBC_GFX].u1IIR_TH[2] = (UINT8)(u4GFX_IIR_TH>>16 & 0x0FF);
    DbcParam_IIR[E_DBC_GFX].u1IIR_TH[3] = (UINT8)(u4GFX_IIR_TH>>24 & 0x0FF);
    DbcParam_IIR[E_DBC_GFX].u1IIR_WEI[0] = (UINT8)(u4GFX_IIR_WEI & 0x0FF);
    DbcParam_IIR[E_DBC_GFX].u1IIR_WEI[1] = (UINT8)(u4GFX_IIR_WEI>>8 & 0x0FF);
    DbcParam_IIR[E_DBC_GFX].u1IIR_WEI[2] = (UINT8)(u4GFX_IIR_WEI>>16 & 0x0FF);
}

static void _vDRVCUST_DBC_Calc_Param_Update(void)
{
    UINT32 u4APL_TH_WEI = u4IO32Read4B(DBC_PARAM1_APL_01);
    UINT32 u4BK_TH_WEI = u4IO32Read4B(DBC_PARAM1_BK_01);
    UINT32 u4MAX_DET_TH_WEI = u4IO32Read4B(DBC_PARAM1_MAX_DET_01);

    // APL
    DbcParam_CALC[E_DBC_APL].u1Enable =  IO32ReadFldAlign(DBC_PARAM1_APL_00, DBC_APL_EN);
    DbcParam_CALC[E_DBC_APL].u1TH[0] = (UINT8)(u4APL_TH_WEI & 0x0FF);
    DbcParam_CALC[E_DBC_APL].u1TH[1] = (UINT8)(u4APL_TH_WEI>>8 & 0x0FF);
    DbcParam_CALC[E_DBC_APL].u1WEI[0] = (UINT8)(u4APL_TH_WEI>>16 & 0x0FF);
    DbcParam_CALC[E_DBC_APL].u1WEI[1] = 0;

    // BK
    DbcParam_CALC[E_DBC_BK].u1Enable =  IO32ReadFldAlign(DBC_PARAM1_BK_00, DBC_BK_EN);
    DbcParam_CALC[E_DBC_BK].u1TH[0] = (UINT8)(u4BK_TH_WEI & 0x0FF);
    DbcParam_CALC[E_DBC_BK].u1TH[1] = (UINT8)(u4BK_TH_WEI>>8 & 0x0FF);
    DbcParam_CALC[E_DBC_BK].u1WEI[0] = 0;
    DbcParam_CALC[E_DBC_BK].u1WEI[1] = (UINT8)(u4BK_TH_WEI>>16 & 0x0FF);

    // MAX_DET
    DbcParam_CALC[E_DBC_MAX_DET].u1Enable =  IO32ReadFldAlign(DBC_PARAM1_MAX_DET_00, DBC_MAX_DET_EN);
    DbcParam_CALC[E_DBC_MAX_DET].u1TH[0] = (UINT8)(u4MAX_DET_TH_WEI & 0x0FF);
    DbcParam_CALC[E_DBC_MAX_DET].u1TH[1] = (UINT8)(u4MAX_DET_TH_WEI>>8 & 0x0FF);
    DbcParam_CALC[E_DBC_MAX_DET].u1WEI[0] = 255;
    DbcParam_CALC[E_DBC_MAX_DET].u1WEI[1] = 0;

    // GFX
    DbcParam_CALC[E_DBC_GFX].u1Enable =  IO32ReadFldAlign(DBC_PARAM1_GFX_00, DBC_GFX_EN);
}


static void _vDRVCUST_DBC_Reg_Update(void)
{    
    DBC_Reg.u1UI_BL_Level = 0xFF;
    DBC_Reg.u1UI_PS_Mode = 0xFF;

    DBC_Reg.u1LUMA_CORRECT = IO32ReadFldAlign(BL_CONTROL_MAIN_01, LUMA_CORRECT);
    DBC_Reg.u2CURRENT_MIN = IO32ReadFldAlign(BL_CONTROL_MAIN_03, CURRENT_MIN);
    DBC_Reg.u2CURRENT_MAX= IO32ReadFldAlign(BL_CONTROL_MAIN_03, CURRENT_MAX);
    DBC_Reg.u2PWM_RATIO_BLIM = IO32ReadFldAlign(BL_CONTROL_MAIN_01, PWM_RATIO_BLIM);
    DBC_Reg.u2PWM_FRQ = IO32ReadFldAlign(BL_CONTROL_MAIN_02, BL_IPREF_FRQ);

    DBC_Reg.u1SBC_BL_PSH = IO32ReadFldAlign(SBC_PARAM_00, SBC_BL_PSH);
    DBC_Reg.u1SBC_BL_TOP = IO32ReadFldAlign(SBC_PARAM_01, SBC_BL_TOP);
    DBC_Reg.u1SBC_BL_MIDDLE = IO32ReadFldAlign(SBC_PARAM_01, SBC_BL_MIDDLE);
    DBC_Reg.u1SBC_BL_BOTTOM = IO32ReadFldAlign(SBC_PARAM_01, SBC_BL_BOTTOM);
    DBC_Reg.u1SBC_BL_SETUDEN = IO32ReadFldAlign(SBC_PARAM_01, SBC_BL_SETUDEN);

	/* marked because spec update
	DBC_Reg.u1SBC_BL_PSH_CARBAT= IO32ReadFldAlign(SBC_PARAM_02, SBC_BL_PSH_CARBAT);
	DBC_Reg.u1SBC_BL_PSH_PWRBANK= IO32ReadFldAlign(SBC_PARAM_02, SBC_BL_PSH_PWRBANK);
	DBC_Reg.u1SBC_BL_PSH_NORM= IO32ReadFldAlign(SBC_PARAM_02, SBC_BL_PSH_NORM);
	*/

	DBC_Reg.u1SBC_BL_PSH_CARBAT= FlashPanel_GetBLPSHCARBAT();
	DBC_Reg.u1SBC_BL_PSH_PWRBANK= FlashPanel_GetBLPSHPWRBANK();
	DBC_Reg.u1SBC_BL_PSH_NORM= FlashPanel_GetBLPSHNORM();

    DBC_Reg.u1DBC_PS_LUMA_TH_DIG = IO32ReadFldAlign(DBC_PS_PARAM_00, DBC_PS_LUMA_TH_DIG);
    DBC_Reg.u1DBC_PS_LUMA_TH_ANA = IO32ReadFldAlign(DBC_PS_PARAM_00, DBC_PS_LUMA_TH_ANA);
    DBC_Reg.u1DBC_CRI_LUMA_TH = IO32ReadFldAlign(DBC_PS_PARAM_00, DBC_CRI_LUMA_TH);
    DBC_Reg.u1DBC_PS_SAT_TH = IO32ReadFldAlign(DBC_PS_PARAM_00, DBC_PS_SAT_TH);
    DBC_Reg.u2DBC_PS_TGT = IO32ReadFldAlign(DBC_PS_PARAM_01, DBC_PS_TGT);
    DBC_Reg.u1DBC_PS_TM_ENTER = IO32ReadFldAlign(DBC_PS_PARAM_01, DBC_PS_TM_ENTER);
    DBC_Reg.u1DBC_PS_TM_EXIT = IO32ReadFldAlign(DBC_PS_PARAM_01, DBC_PS_TM_EXIT);
    DBC_Reg.u1DBC_PS_ATN_DOWN = IO32ReadFldAlign(DBC_PS_PARAM_02, DBC_PS_ATN_DOWN);
    DBC_Reg.u1DBC_PS_ATN_UP = IO32ReadFldAlign(DBC_PS_PARAM_02, DBC_PS_ATN_UP);

    DBC_Reg.u1BLINK_GAIN_TGT = IO32ReadFldAlign(BLINK_PARAM, BLINK_GAIN_TGT);    
    DBC_Reg.u1BLINK_PHASE = IO32ReadFldAlign(BLINK_PARAM, BLINK_PHASE);
    DBC_Reg.u1BLINK_DUTY = IO32ReadFldAlign(BLINK_PARAM, BLINK_DUTY);
}


void _vDRVCUST_DBLProc(void)
{
    static UINT16 u2sbcGain=0,u2currentGain=0,u2PWM_RATIO2=0,u2CURRENT_GAIN=0;
    static UINT16 u2BL_DBC=0, u2DBC_GAIN_TOTAL=0;
    static UINT8  u1RawSbcGain=0x80;
    static UINT8  fgDblHw = SV_OFF;
    UINT8  bEEIOnOff = IO32ReadFldAlign(EEI_PARAM_00,DBC_EEI_ON);
    UINT16 u2BL_POWER_LIMIT_GAIN,u2sbcPowerGain,u2sbcPowerGain1,u2sbcPowerGain2;
    UINT16 wDimmer_out = 0x200;
    UINT8  u1SigStatus = bDrvVideoSignalStatus(VDP_1);
    UINT16 u2PWM_RATIO1=0,u2PWM_RATIO_SUB=0,u2PWM_RATIO=0;
    UINT8  i,u1ASL;
    UINT16 wLumaMax;
    UINT8  u1APL,u1DbcMaxDetTh,u1DBC_GFX_TGT;
	//Below added for car bank
	static UINT16 u2Bl_Carbat_Gain=GAIN_BASE,u2Bl_PWRBank_Gain=GAIN_BASE;
	UINT16 u2_Blcarbat_TGT,u2_BlPWRBank_TGT;
	UINT8 u1PWR_Type;
	static UINT8 u1Pre_PWR_Type=0xEE;//just for init value
	
	u2_Blcarbat_TGT=FlashPanel_GetBLCARBATTGT();
	u2_BlPWRBank_TGT=FlashPanel_GetBLPWRBANKTGT();
	u1PWR_Type=SelfDiagGetPowerMode();//IO32ReadFldAlign(DBC_PARAM1_MAIN, DBC_POWER_BANK_PWR_TYPE);
	//u1PWR_Type=IO32ReadFldAlign(DBC_PARAM1_MAIN, DBC_POWER_BANK_PWR_TYPE);
	//u2Bl_Carbat_Gain=GAIN_BASE;
	//u2Bl_PWRBank_Gain=GAIN_BASE;
	//above added for car bank

    // ---Get OSD status---
    fg_OSD_Exist = OSD_BASE_SonyIsDisplaying();
    vIO32WriteFldAlign(BL_CONTROL_MAIN_01, fg_OSD_Exist, OSD_EXIST_STATUS);
    
    if(IO32ReadFldAlign(BL_CONTROL_MAIN_00, PARAM_UPDATE_PER_V))  //For debug, Update param evey Vsync ?
    {    
        _vDRVCUST_DBC_Reg_Update();
        if(DBC_Reg.u1DBC_EN)
        {
            _vDRVCUST_DBC_IIR_Param_Update();
        }
    }
    DBC_Reg.u1DBC_EN = IO32ReadFldAlign(BL_CONTROL_MAIN_00, DBC_EN);
    DBC_Reg.u1DBC_PS_EN = IO32ReadFldAlign(BL_CONTROL_MAIN_00, DBC_PS_EN);
    DBC_Reg.u1DBC_CRI_EN = IO32ReadFldAlign(BL_CONTROL_MAIN_00, DBC_CRI_EN);
    DBC_Reg.u1ZEROD_EN = IO32ReadFldAlign(BL_CONTROL_MAIN_00, ZEROD_EN);
    DBC_Reg.u1BLINK_EN = IO32ReadFldAlign(BL_CONTROL_MAIN_00, BLINK_EN);
    
    //-------------------------------------------------------------------
    //---[1]. SBC Part
    //-------------------------------------------------------------------
    if((DBC_Reg.u1UI_BL_Level != IO32ReadFldAlign(BL_CONTROL_MAIN_01, UI_BL_LEVEL))
        ||(DBC_Reg.u1UI_PS_Mode != IO32ReadFldAlign(BL_CONTROL_MAIN_01, UI_BL_PS_MODE))||(u1PWR_Type!=u1Pre_PWR_Type))
    {
        DBC_Reg.u1UI_BL_Level = IO32ReadFldAlign(BL_CONTROL_MAIN_01, UI_BL_LEVEL);
        DBC_Reg.u1UI_PS_Mode = IO32ReadFldAlign(BL_CONTROL_MAIN_01, UI_BL_PS_MODE);
        switch(DBC_Reg.u1UI_PS_Mode)
        {
            case 3: //Mode = 3 (Power Saving = High)

				if (u1PWR_Type==1)
					{
					DBC_Reg.u1SBC_BL_PSH=DBC_Reg.u1SBC_BL_PSH_CARBAT;
					}
				else if (u1PWR_Type==2)
					{
					DBC_Reg.u1SBC_BL_PSH=DBC_Reg.u1SBC_BL_PSH_PWRBANK;
					}
				else
					{
					DBC_Reg.u1SBC_BL_PSH=DBC_Reg.u1SBC_BL_PSH_NORM;
					}
				DBC_Reg.u1SBC_GAIN =  (INT16)DBC_Reg.u1SBC_BL_PSH;
				LOG(0,"DBC_Reg.u1SBC_GAIN=%d in case PWS H...\n",DBC_Reg.u1SBC_GAIN);
				break;
				
            case 4: //Mode = 4 (Power Saving = PICTURE OFF)
                DBC_Reg.u1SBC_GAIN =  (INT16)IO32ReadFldAlign(SBC_PARAM_00, SBC_BL_PSH);             
                break;
            case 0: //Mode = 0 (Power Saving = OFF)
            case 1: //Mode = 1 (Power Saving = LOW)
            default:
                if(DBC_Reg.u1UI_BL_Level >= 5)
                {
                    DBC_Reg.u1SBC_GAIN = (UINT8)((((INT16)DBC_Reg.u1SBC_BL_TOP - DBC_Reg.u1SBC_BL_MIDDLE)*(DBC_Reg.u1UI_BL_Level-5))/5
                                                 + DBC_Reg.u1SBC_BL_MIDDLE);
                }
                else
                {
                    DBC_Reg.u1SBC_GAIN = (UINT8)((((INT16)DBC_Reg.u1SBC_BL_MIDDLE - DBC_Reg.u1SBC_BL_BOTTOM)*DBC_Reg.u1UI_BL_Level)/5
                                                 + DBC_Reg.u1SBC_BL_BOTTOM);
                }

                if(DBC_Reg.u1UI_PS_Mode == 1)    // Mode=1(LOW) : Minus BL_STUEN 
                {
                    DBC_Reg.u1SBC_GAIN = DBC_Reg.u1SBC_GAIN-DBC_Reg.u1SBC_BL_SETUDEN;
                }
                break;			
		
        }
		
        u1RawSbcGain = DBC_Reg.u1SBC_GAIN;
		
		//--begain patch for car bank	
		if (u1PWR_Type==1)
			{
				LOG(1,"/////////POWER BANK set (u1PWR_Type=1)?,u1PWR_Type=%d\n",u1PWR_Type);
				u2Bl_Carbat_Gain=u2_Blcarbat_TGT;
				u2Bl_PWRBank_Gain=GAIN_BASE;
		
			}
		else if (u1PWR_Type==2)
			{
				LOG(1,"/////////POWER BANK set (u1PWR_Type=2)?,u1PWR_Type=%d\n",u1PWR_Type);
				u2Bl_Carbat_Gain=GAIN_BASE;
				u2Bl_PWRBank_Gain=u2_BlPWRBank_TGT;
			}
		else
			{
				LOG(1,"/////////POWER BANK set (u1PWR_Type=else)?,u1PWR_Type=%d\n",u1PWR_Type);
				u2Bl_Carbat_Gain=GAIN_BASE;
				u2Bl_PWRBank_Gain=GAIN_BASE;
			}
			LOG(0,"DBC_Reg.u1SBC_GAIN=%d...\n",DBC_Reg.u1SBC_GAIN);
			LOG(0,"/////////POWER BANK get u2Bl_Carbat_Gain=%d\n",u2Bl_Carbat_Gain);
			LOG(0,"/////////POWER BANK get u2Bl_PWRBank_Gain=%d\n",u2Bl_PWRBank_Gain);
			u1Pre_PWR_Type=u1PWR_Type;
		//--end patch for car bank
		
    }

    //--Begain patch for EEI--
    if(bEEIOnOff)
    {   
        DBC_Reg.u1SBC_GAIN = DRVCUST_EEIModify((UINT16)u1RawSbcGain);
        if(IO32ReadFldAlign(EEI_PARAM_00, DBC_EEI_DEBUG_ON)==1)
        {
            printf("==EEI OriDimming[%d] --> AfterDimming[%d]==\n", 
                    u1RawSbcGain,DBC_Reg.u1SBC_GAIN);
        }
    }
    //--End patch for EEI--
    
    u2sbcGain = ((UINT32)DBC_Reg.u1SBC_GAIN*GAIN_BASE+0)/255;
    u2BL_POWER_LIMIT_GAIN = u2DRVCUST_DBC_Get_Power_limit_gain();
    u2sbcPowerGain1 = ((UINT32)u2sbcGain * u2BL_POWER_LIMIT_GAIN+0)>>10;
	//below added for car bank 
	u2sbcPowerGain2 = ((UINT32)u2sbcPowerGain1*u2Bl_Carbat_Gain)>>10;
	u2sbcPowerGain = ((UINT32)u2sbcPowerGain2*u2Bl_PWRBank_Gain)>>10;
	//above added for car bank 

    u2currentGain = MIN(1024,MAX(u2sbcPowerGain,((UINT32)DBC_Reg.u2CURRENT_MIN*GAIN_BASE/MAX(DBC_Reg.u2CURRENT_MAX,1))));
    u2PWM_RATIO2 = (UINT32)u2sbcPowerGain * GAIN_BASE/MAX(u2currentGain,1);
    u2CURRENT_GAIN = (UINT32)u2currentGain * DBC_Reg.u2CURRENT_MAX/GAIN_BASE;
    DBC_Reg.u1IP_REF_Dimming = ((MIN(u2CURRENT_GAIN,1023)+2)>>2);

    if(IO32ReadFldAlign(DBC_DEBUG_USE, DBC_DEBUG_LOG_00))
    {
    	printf("---[sbcg]%d,[PWTYP]%d [CURMIN]%d,[CURMAX]%d,[SBCG]%d,[SBCPG]%d,[CG]%d,[CG2]%d,[PWMR2]%d,[BLPPL]%d ,[CarbatG]%d,[PWRBankG]%d---\n",DBC_Reg.u1SBC_GAIN,u1PWR_Type,DBC_Reg.u2CURRENT_MIN,DBC_Reg.u2CURRENT_MAX,u2sbcGain,u2sbcPowerGain,u2currentGain,u2CURRENT_GAIN,u2PWM_RATIO2,u2BL_POWER_LIMIT_GAIN,u2Bl_Carbat_Gain,u2Bl_PWRBank_Gain);
	}

    vIO32WriteFldAlign(DBC_MISC_00, u2sbcGain, M_sbcGain);
    vIO32WriteFldAlign(DBC_MISC_00, u2currentGain, M_currentGain);
    vIO32WriteFldAlign(DBC_MISC_01, u2CURRENT_GAIN, M_CURRENT_GAIN);
    vIO32WriteFldAlign(DBC_MISC_01, u2PWM_RATIO2, M_PWM_RATIO2);
    vIO32WriteFldAlign(DBC_MISC_05, u2BL_POWER_LIMIT_GAIN, M_BL_POWER_LIMIT_GAIN);
    vIO32WriteFldAlign(DBC_MISC_05, u2sbcPowerGain, M_sbcPowerGain);

    vIO32WriteFldAlign(SBC_PARAM_00, DBC_Reg.u1SBC_GAIN, SBC_GAIN);
    vIO32WriteFldAlign(BL_CONTROL_MAIN_00, DBC_Reg.u1IP_REF_Dimming, IP_REF_DIMMING);

    //-------------------------------------------------------------------
    //---[2]. DBC Part
    //-------------------------------------------------------------------
    DBC_Reg.u1DBC_GAIN = 0;
    if(DBC_Reg.u1DBC_EN)
    {
        // <a>  Load related parameter
        u1APL = bDrvGetAPL();
        u1APL = (u1APL < 16) ? 16 : ((u1APL > 235) ? 235 : u1APL);
        u1DbcMaxDetTh = IO32ReadFldAlign(DBC_PARAM0_MAX_DET_01, DBCP_MAX_DET_TH);
        u1DBC_GFX_TGT = IO32ReadFldAlign(DBC_PARAM0_GFX_01, DBCP_GFX_TGT);

        DbcParam_IIR[E_DBC_APL].u1In = u1APL;    //Load APL
        DbcParam_IIR[E_DBC_BK].u1In = _u1DrvDBC_GetBlackIndex();    //Load Black Index
        DbcParam_IIR[E_DBC_MAX_DET].u1In = _u1DrvDBC_Get_MaxBrigtness(u1DbcMaxDetTh);    //Load MaxBrightness
        DbcParam_IIR[E_DBC_GFX].u1In = fg_OSD_Exist ?  u1DBC_GFX_TGT : 255;    //Load GFX
    
        _vDRVCUST_DBC_Calc_Param_Update();
        
        // <b>  Calculate APL_DBC,BK_DBC,MAX_DBC,DBC_GFX
        for(i=E_DBC_APL; i<E_DBC_MAX; i++)
        {
            DbcParam_IIR[i].u1AbsDiff = AbsDiff(DbcParam_IIR[i].u1In, DbcParam_IIR[i].u1Result);
            DbcParam_IIR[i].u1Coef    = _u1Util_math_step_func(DbcParam_IIR[i].u1AbsDiff, DbcParam_IIR[i].u1IIR_TH, DbcParam_IIR[i].u1IIR_WEI);
			if(u1SigStatus != SV_VDO_STABLE)
			{
				DbcParam_IIR[i].u1Coef = 255;
			}
			DbcParam_IIR[i].u1Result  = _u1DrvDBC_Util_IIR(i, DbcParam_IIR[i].u1Coef, DbcParam_IIR[i].u1In, DbcParam_IIR[i].u1Result);
            if(i<=E_DBC_MAX_DET)
            {
                DbcParam_CALC[i].u1Gain_Result = _u1DrvDBC_Calc_Gain(DbcParam_IIR[i].u1Result,DbcParam_CALC[i].u1TH,DbcParam_CALC[i].u1WEI);
            }
        }
        DbcParam_CALC[E_DBC_APL].u1Gain_Result = DbcParam_CALC[E_DBC_APL].u1Enable ? 
                                                    DbcParam_CALC[E_DBC_APL].u1Gain_Result : 0;
        DbcParam_CALC[E_DBC_BK].u1Gain_Result = DbcParam_CALC[E_DBC_BK].u1Enable ? 
                                                    DbcParam_CALC[E_DBC_BK].u1Gain_Result : 0;
        DbcParam_CALC[E_DBC_MAX_DET].u1Gain_Result = DbcParam_CALC[E_DBC_MAX_DET].u1Enable ? 
                                                        DbcParam_CALC[E_DBC_MAX_DET].u1Gain_Result : 255;
        DbcParam_CALC[E_DBC_GFX].u1Gain_Result = DbcParam_CALC[E_DBC_GFX].u1Enable ? 
                                                    DbcParam_IIR[E_DBC_GFX].u1Result : 255;

        //<c> Calculate DBC_GAIN
        if(wReadQualityTable(QUALITY_AL_ON_1_OFF_0) != SV_OFF)
        {
            DBC_Reg.u1DBC_GAIN = (((((UINT32)DbcParam_CALC[E_DBC_APL].u1Gain_Result + DbcParam_CALC[E_DBC_BK].u1Gain_Result)
                                    * DbcParam_CALC[E_DBC_MAX_DET].u1Gain_Result * DbcParam_CALC[E_DBC_GFX].u1Gain_Result) 
                                     + (1<<15))>>16);
        }

        //<d> Save Result to sw register
        vIO32WriteFldMulti(DBC_PARAM1_APL_00, P_Fld(DbcParam_IIR[E_DBC_APL].u1Result,APL_DBC)
                                              |P_Fld(DbcParam_CALC[E_DBC_APL].u1Gain_Result,DBC_APL_GAIN));
        vIO32WriteFldMulti(DBC_PARAM1_BK_00, P_Fld(DbcParam_IIR[E_DBC_BK].u1Result,BK_DBC)
                                              |P_Fld(DbcParam_CALC[E_DBC_BK].u1Gain_Result,DBC_BK_GAIN));
        vIO32WriteFldMulti(DBC_PARAM1_MAX_DET_00, P_Fld(DbcParam_IIR[E_DBC_MAX_DET].u1Result,MAX_DET_DBC)
                                              |P_Fld(DbcParam_CALC[E_DBC_MAX_DET].u1Gain_Result,DBC_MAX_DET_GAIN));
        vIO32WriteFldMulti(DBC_PARAM1_GFX_00, P_Fld(DbcParam_IIR[E_DBC_GFX].u1Result,GFX_DBC)
                                              |P_Fld(DbcParam_CALC[E_DBC_GFX].u1Gain_Result,DBC_GFX_GAIN));
    }
    vIO32WriteFldAlign(DBC_PARAM1_MAIN, DBC_Reg.u1DBC_GAIN, DBC_GAIN);

    
    //-------------------------------------------------------------------
    //---[3]. Dynamic Backlight Control Blending/0DD Blending
    //-------------------------------------------------------------------
    DBC_Reg.u1BLINK_GAIN = (DRVCUST_GetLEDMotionFlag() && DBC_Reg.u1BLINK_EN) ? DBC_Reg.u1BLINK_GAIN_TGT : 255;
    DBC_Reg.u1HEAT_BLSAVE = (/*(wReadQualityTable(QUALITY_AL_ON_1_OFF_0) != SV_OFF) &&*/ DBC_Reg.u1ZEROD_EN ) ? 
                                IO32ReadFldAlign(OPC_0C, OPC_SW_PWM) : 255;
    DBC_Reg.u1HEAT_BLSAVE_GAIN = (/*(wReadQualityTable(QUALITY_AL_ON_1_OFF_0) != SV_OFF) &&*/ (u1PWR_Type!=1) && (u1PWR_Type!=2)) ? 
									IO32ReadFldAlign(ZEROD_DIMMING_PARAM, ZEROD_HEAT_BLSAVE_GAIN) : 128;
    // for PCC spec use    
    //u2BL_POWER_REQ = (u2CURRENT_GAIN * DBC_Reg.u1BLINK_GAIN)/DBC_Reg.u2CURRENT_MAX;

    u2BL_DBC = MAX(0,((255-DBC_Reg.u1DBC_GAIN)*GAIN_BASE/255));
    u2DBC_GAIN_TOTAL = (((UINT32)u2PWM_RATIO2 * u2currentGain*DBC_Reg.u1BLINK_GAIN)+(1<<17))>>18;
    u2PWM_RATIO1 = u2BL_DBC +((((UINT32)(GAIN_BASE-u2DBC_GAIN_TOTAL)*
                   (1024-u2BL_DBC)*DBC_Reg.u1LUMA_CORRECT)+(1<<17))>>18);
    u2PWM_RATIO_SUB = ((UINT32)u2PWM_RATIO1 * u2PWM_RATIO2+512)>>10;

    u2PWM_RATIO = ((wReadQualityTable(QUALITY_AL_ON_1_OFF_0) != SV_OFF) && DBC_Reg.u1ZEROD_EN && (u1SigStatus == SV_VDO_STABLE)) ? 
                    MIN(((DBC_Reg.u1HEAT_BLSAVE<<2)*DBC_Reg.u1HEAT_BLSAVE_GAIN*2)>>8, 1024) : u2PWM_RATIO_SUB; //((UINT32)u2PWM_RATIO_SUB * (DBC_Reg.u1HEAT_BLSAVE+1))>>8; //0DD Blending
	
	if (u1PWR_Type==2)
	{
		if (_bDemoMode == TRUE) //keep 70% backlight in demo mode
		{
			GPIO_SetOut(GPIO(57), 0);
        	u2PWM_RATIO =  179*4;    //70%
		}
		else
		{
			if (DBC_Reg.u1UI_PS_Mode == 3)
			{
				GPIO_SetOut(GPIO(57), 1);
				u2PWM_RATIO =  26*4;
			}
			else
			{
				GPIO_SetOut(GPIO(57), 0);
			}
		}
				
	}
    // Note below :  'no signal' or 'mute condition' or 'photo frame mode' ,  fix backlight to 76%.
    if((u1SigStatus != SV_VDO_STABLE) ||
        (_u4DrvVideoGetMute(SV_VP_MAIN) == SV_TRUE) ||
        (u2OpcUnMuteCount <= IO32ReadFldAlign(ZEROD_PATCH_00, ZEROD_UNMUTE_DELAY)) ||
        (DRVCUST_GetPhotoFrameMode() == SV_TRUE) ||
        (bEPGFlag == SV_TRUE) ||
        (bTeleText_ONOFF == SV_TRUE) ||
        (bMHEG_ONOFF == SV_TRUE) ||
        (bGinga_ONOFF == SV_TRUE))
    {
        //u2PWM_RATIO = 193*4;
        if (u1PWR_Type==2)
		{
			u2PWM_RATIO = (bGinga_ONOFF == SV_TRUE) ? (255*4) : (26*4);
		}
		else if (u1PWR_Type==1)
		{
			u2PWM_RATIO = (bGinga_ONOFF == SV_TRUE) ? (255*4) : (51*4);
		}
		else
		{
        	u2PWM_RATIO = (bGinga_ONOFF == SV_TRUE) ? (255*4) : (193*4);
		}
        fgDblHw = SV_OFF;
        vIO32WriteFldAlign(DBL_00, SV_OFF, DBL_ENABLE);
    }
    else if (fgDblHw == SV_OFF)
    {
        fgDblHw = SV_ON;
        vIO32WriteFldAlign(DBL_00, SV_ON, DBL_ENABLE);
    }
    
    wDimmer_out = DBC_CLIP(u2PWM_RATIO, DBC_Reg.u2PWM_RATIO_BLIM, 1023);
    DBC_Reg.u1PWM_Dimming = wDimmer_out>>2;

    vIO32WriteFldAlign(ZEROD_DIMMING_PARAM, DBC_Reg.u1HEAT_BLSAVE, ZEROD_HEAT_BLSAVE);
    vIO32WriteFldAlign(BL_CONTROL_MAIN_00, DBC_Reg.u1BLINK_GAIN, BLINK_GAIN);
    vIO32WriteFldAlign(BL_CONTROL_MAIN_02, DBC_Reg.u1PWM_Dimming, DIMMER1_OUT);
    vIO32WriteFldAlign(DBC_MISC_02, u2BL_DBC, M_BL_DBC);
    vIO32WriteFldAlign(DBC_MISC_02, u2DBC_GAIN_TOTAL, M_DBC_GAIN_TOTAL);
    vIO32WriteFldAlign(DBC_MISC_03, u2PWM_RATIO1, M_PWM_RATIO1);
    vIO32WriteFldAlign(DBC_MISC_03, u2PWM_RATIO_SUB, M_PWM_RATIO_SUB);
    vIO32WriteFldAlign(DBC_MISC_04, u2PWM_RATIO, M_PWM_RATIO);
    //vIO32WriteFldAlign(DBC_MISC_05, u2BL_POWER_REQ, M_BL_POWER_REQ);

    //---Temp debug---
    if(IO32ReadFldAlign(DBC_DEBUG_USE, DBC_DEBUG_LOG_00))
    {
        Printf("DBC00=%d,%d,%d, [BLKG]%d,[HEATBL]%d,[DBCG]%d, [DBGGTOL]%d,[PWMR1]%d,[PWMRSUB]%d,[pwm]%d,[pwmDim]%d,[IPRef]%d=\n",
            u1SigStatus,u2OpcUnMuteCount,DRVCUST_GetPhotoFrameMode(),
            DBC_Reg.u1BLINK_GAIN,DBC_Reg.u1HEAT_BLSAVE,DBC_Reg.u1DBC_GAIN,
            u2DBC_GAIN_TOTAL,u2PWM_RATIO1,u2PWM_RATIO_SUB,
            u2PWM_RATIO,DBC_Reg.u1PWM_Dimming, DBC_Reg.u1IP_REF_Dimming);
    }

    if(IO32ReadFldAlign(DBC_DEBUG_USE, DBC_DEBUG_LOG_02))
    {
        Printf("DBC02== %d,%d,%d,%d ==\n",
            bEPGFlag,bTeleText_ONOFF,bMHEG_ONOFF,bGinga_ONOFF);
    }
    //---End Temp debug------------

    //-------------------------------------------------------------------
    //---[4]. Update the result
    //-------------------------------------------------------------------

    //-------------------------------------------------------------------
    //For Power On Sequence case
    //if(((fgPWOn_Sequence == TRUE) && (_u4DrvVideoGetMute(SV_VP_MAIN) ==0)) 
    //    ||((fgPWOn_Sequence == TRUE) && ((bUSBPhotoPlayback ==1)||(bFMRadioPlayback ==1))))
    //{
    //    fgPWOn_Sequence = FALSE;
    //}

    if(0)//(fgPWOn_LogoFinish != TRUE)  // 1. For Power On Sequence case
    {
        wDimmer_out = 1023;
        wPreDimmer_out = wDimmer_out;
        DBC_Reg.u1PWM_Dimming_Final = wDimmer_out>>2;
        DBC_Reg.u1DBC_PS_IN_STATE = 0;
        vIO32WriteFldAlign(DBC_PS_PARAM_02, DBC_Reg.u1DBC_PS_IN_STATE, DBC_PS_IN_STATE);
    }       
    else if((u1SigStatus == SV_VDO_NOSIGNAL) || (u1SigStatus == SV_VDO_NOSUPPORT)
        || (!DBC_Reg.u1DBC_EN) || (!DBC_Reg.u1DBC_PS_EN) 
        || (wReadQualityTable(QUALITY_AL_ON_1_OFF_0) == SV_OFF)
        || (fg_OSD_Exist == TRUE))  // 2. Disable CRI  
    {
        DBC_Reg.u1DBC_PS_OPERATE = 0;
        wPreDimmer_out = wDimmer_out;
        DBC_Reg.u1PWM_Dimming_Final = wDimmer_out>>2;
        DBC_Reg.u1DBC_PS_IN_STATE = 0;
        DBC_Reg.u1DBC_PS_PIC_STATE = 0;
        
        vIO32WriteFldAlign(BL_CONTROL_MAIN_00, DBC_Reg.u1PWM_Dimming_Final, FINAL_DIMMER1_OUT);
        vIO32WriteFldAlign(DBC_PS_PARAM_02, DBC_Reg.u1DBC_PS_IN_STATE, DBC_PS_IN_STATE);
        vIO32WriteFldAlign(DBC_PS_PARAM_02, DBC_Reg.u1DBC_PS_OPERATE, DBC_PS_OPERATE);
        vIO32WriteFldAlign(DBC_PS_PARAM_02, DBC_Reg.u1DBC_PS_PIC_STATE, DBC_PS_PIC_STATE);

        if((_bfgCustPowerOnSeqPWMON == FALSE)&&(fgPWMFixed == FALSE))//do real delay T41/T42(>1ms) after power on
        {
            DRVCUST_SetBacklightDuty(VDP_ARG_BACKLT_DBL, DBC_Reg.u1PWM_Dimming_Final);  //DIMMER2(OPWM2)
        }
        
        if((_bfgCustPowerSeqOnOff) && (fgPWOn_LogoFinish) &&
            (DBC_UI.ui1BacklightLevel != 0xFF) && (fgPWMFixed == FALSE))    //TRUE:power on, FLASE:power off
        {
            vDRVCUST_BLSet_IP_REF(DBC_Reg.u1IP_REF_Dimming);  //---IP_REF(OPWM1) 
        }
        
        if(!fg_PS_Backlight_On)
        {
            fg_PS_Backlight_On = TRUE;
            vDrvTurnOnBacklt();
        }

        //For debug
        if(IO32ReadFldAlign(DBC_DEBUG_USE, DBC_DEBUG_LOG_01))
		{
            Printf("DBC11=%d,%d,%d,%d,%d,%d,%d=\n", _bfgCustPowerOnSeqPWMON,fgPWMFixed,DBC_Reg.u1PWM_Dimming_Final,
                _bfgCustPowerSeqOnOff,fgPWOn_LogoFinish,DBC_UI.ui1BacklightLevel,DBC_Reg.u1IP_REF_Dimming);
		}
    }
    else // 3. Enable CRI
    {
        DBC_Reg.u1DBC_PS_OPERATE = 1;
        u1ASL = u1DrvGetASL();
        wLumaMax = IO32ReadFldAlign(LUMA_MIN_MAX_MAIN, SCE_LUMA_MAX);
        if ((DBC_Reg.u1DBC_PS_IN_STATE == 0) && (u1SigStatus == SV_VDO_STABLE)
             && (u1ASL <= DBC_Reg.u1DBC_PS_SAT_TH)
             && (wLumaMax <= DBC_Reg.u1DBC_CRI_LUMA_TH))  //Black ?
        {
            if (u2PS_InCount <= (DBC_Reg.u1DBC_PS_TM_ENTER + 30))
            {
                u2PS_InCount++;
            }
            DBC_Reg.u1DBC_PS_PIC_STATE = 1;
        }
        else
        {
            u2PS_InCount = 0; 
        }

        if ((DBC_Reg.u1DBC_PS_IN_STATE == 1) && ((u1SigStatus != SV_VDO_STABLE)
            || (u1ASL > DBC_Reg.u1DBC_PS_SAT_TH)
            || (wLumaMax > DBC_Reg.u1DBC_CRI_LUMA_TH)))  // !Black ?
        {
            if (u2PS_OutCount < (DBC_Reg.u1DBC_PS_TM_EXIT + 30))
            {
                u2PS_OutCount++;
            }
            DBC_Reg.u1DBC_PS_PIC_STATE = 0;
        }
        else
        {
            u2PS_OutCount = 0;    
        }

        switch(DBC_Reg.u1DBC_PS_IN_STATE)
        {
            case 0: //Not in DBC4PS
            default:
                if (u2PS_InCount > DBC_Reg.u1DBC_PS_TM_ENTER)
                {
                    DBC_Reg.u1DBC_PS_IN_STATE = 1;                
                }
                if (fg_PS_TriggerMODE == 1)
                {
                    if (wPreDimmer_out < wDimmer_out)
                    {
                        wDimmer_out = (wPreDimmer_out + DBC_Reg.u1DBC_PS_ATN_UP< 0x3FF)?
                                       wPreDimmer_out + DBC_Reg.u1DBC_PS_ATN_UP:0x3FF;                    
                        if(fgPWMFixed == FALSE)
                        {
                            DRVCUST_SetBacklightDuty(VDP_ARG_BACKLT_DBL, (UINT8)(wDimmer_out>>2));
                            vDRVCUST_BLSet_IP_REF(DBC_Reg.u1IP_REF_Dimming);  //---IP_REF(OPWM1) 
                        }
                    }
                    else
                    {
                        if(fgPWMFixed == FALSE)
                        {
                            DRVCUST_SetBacklightDuty(VDP_ARG_BACKLT_DBL, (UINT8)(wDimmer_out>>2));
                            vDRVCUST_BLSet_IP_REF(DBC_Reg.u1IP_REF_Dimming);  //---IP_REF(OPWM1) 
                        }
                        fg_PS_TriggerMODE = 0;
                    }
                    if(!fg_PS_Backlight_On)
                    {
                        fg_PS_Backlight_On = TRUE;
                        vDrvTurnOnBacklt();  // mark million to 1 function to follow PAA 
                    }
                 }
                 else
                 {
                    if(fgPWMFixed == FALSE)
                    {    
                        DRVCUST_SetBacklightDuty(VDP_ARG_BACKLT_DBL, (UINT8)(wDimmer_out>>2));
                        vDRVCUST_BLSet_IP_REF(DBC_Reg.u1IP_REF_Dimming);  //---IP_REF(OPWM1) 
                    }
                 }       
                 break;
            case 1: //In DBC4PS
                fg_PS_TriggerMODE = 1;
                if (u2PS_OutCount > DBC_Reg.u1DBC_PS_TM_EXIT)
                {
                    DBC_Reg.u1DBC_PS_IN_STATE = 0;
                }
                if (wPreDimmer_out > DBC_Reg.u2DBC_PS_TGT)
                {
                    wDimmer_out = MAX(DBC_Reg.u2DBC_PS_TGT,wPreDimmer_out - DBC_Reg.u1DBC_PS_ATN_DOWN); 
                    if(fgPWMFixed == FALSE)
                    {
                        DRVCUST_SetBacklightDuty(VDP_ARG_BACKLT_DBL, (UINT8)(wDimmer_out>>2));
                        vDRVCUST_BLSet_IP_REF(DBC_Reg.u1IP_REF_Dimming);  //---IP_REF(OPWM1) 
                    }
                }
                else
                {
                    wDimmer_out = DBC_Reg.u2DBC_PS_TGT;
                    if((fg_OSD_Exist == FALSE) && (DBC_Reg.u1DBC_CRI_EN == 1) /*&& PS_Backlight_OnOff*/)  //only when no osd, when OSD show, exit CRI
                    {
                        fg_PS_Backlight_On = FALSE;
                        
                        if(fgPWMFixed == FALSE)
                        {
                            vDrvTurnOffBacklt();                            
                            DRVCUST_SetBacklightDuty(VDP_ARG_BACKLT_DBL, (UINT8)0x0);
                            vDRVCUST_BLSet_IP_REF(0);  //---IP_REF(OPWM1)
                        }
                    }
                }
                break;
        }
        wPreDimmer_out = wDimmer_out;
        DBC_Reg.u1PWM_Dimming_Final = wDimmer_out>>2;

        vIO32WriteFldAlign(BL_CONTROL_MAIN_00, DBC_Reg.u1PWM_Dimming_Final, FINAL_DIMMER1_OUT);
        vIO32WriteFldAlign(DBC_PS_PARAM_02, DBC_Reg.u1DBC_PS_IN_STATE, DBC_PS_IN_STATE);
        vIO32WriteFldAlign(DBC_PS_PARAM_02, DBC_Reg.u1DBC_PS_OPERATE, DBC_PS_OPERATE);
        vIO32WriteFldAlign(DBC_PS_PARAM_02, DBC_Reg.u1DBC_PS_PIC_STATE, DBC_PS_PIC_STATE);
    }

    // DBC Coef Reset count 
    u1Coef_ResetCount = (u1Coef_ResetCount > 0) ? (u1Coef_ResetCount-1) : 0;
}

void DRVCUST_AdaptiveBacklightInit(void)
{
    // ------------------------------------------------------------------------------------------------------------
    // this function defines the initialization of backlight algorithm
    // for example : initialize IIR strength, select default mapping table, defines backlight lower/upper bound, etc
    // you could also set initialzed backlight value in this function, in case no value in initialization
    // ------------------------------------------------------------------------------------------------------------    
    //EXTERN UINT32 FlashPanel_GetInchSize(void);
    //EXTERN UINT8 CustomPCBAQueryShipRegion(VOID);
    EXTERN UINT32 PANEL_GetPanelWidth(void);

    //Control and Misc
    UINT32 u4CURRENT_MIN = PANEL_Get_DBC_Current_Min();
    UINT32 u4CURRENT_MAX = PANEL_Get_DBC_Current_Max();
    UINT32 u4BL_IPREF_FRQ = PANEL_Get_DBC_CurrentRatio_Freq();
    UINT32 u4BL_PWM_FRQ = BACKLT_GetDimmingFrequency();
    
    DBC_UI.ui1BacklightLevel = 0xFF;    //For power on, logo flicker issue
    vIO32WriteFldAlign(BL_CONTROL_MAIN_00, 1, BL_UPDATE_EN);
    vIO32WriteFldAlign(BL_CONTROL_MAIN_00, 1, DBC_EN);
    vIO32WriteFldAlign(BL_CONTROL_MAIN_00, 1, DBC_PS_EN);
    vIO32WriteFldAlign(BL_CONTROL_MAIN_00, 1, DBC_CRI_EN);
    vIO32WriteFldAlign(BL_CONTROL_MAIN_00, 0, ZEROD_EN);
    vIO32WriteFldAlign(BL_CONTROL_MAIN_00, 0, BLINK_EN);
    vIO32WriteFldAlign(BL_CONTROL_MAIN_00, 0, PARAM_UPDATE_PER_V);
    
    vIO32WriteFldAlign(BL_CONTROL_MAIN_01, 128, LUMA_CORRECT);      //32
    vIO32WriteFldAlign(BL_CONTROL_MAIN_01, 72, PWM_RATIO_BLIM);    //204
    
    vIO32WriteFldAlign(BL_CONTROL_MAIN_03, 
                          ((u4CURRENT_MIN==0) ? 84 : u4CURRENT_MIN), CURRENT_MIN);//====TBD===
    vIO32WriteFldAlign(BL_CONTROL_MAIN_03, 
                          ((u4CURRENT_MAX==0) ? 280 : u4CURRENT_MAX), CURRENT_MAX);//====TBD===

  /*
    //-- Patch for india 40 inch
    if((FlashPanel_GetInchSize()==40)&&(CustomPCBAQueryShipRegion()==3))
    {
        vIO32WriteFldAlign(BL_CONTROL_MAIN_03, 363 , CURRENT_MAX);
    }
    //--End path for inida 40 inch
    */
                          
    vIO32WriteFldAlign(BL_CONTROL_MAIN_02, 
                          ((u4BL_IPREF_FRQ==0) ? 80000 : u4BL_IPREF_FRQ), BL_IPREF_FRQ);
    vIO32WriteFldAlign(BL_CONTROL_MAIN_04, 
                          ((u4BL_PWM_FRQ==0) ? 1300 : u4BL_PWM_FRQ), BL_PWM_FRQ);
                          
    vIO32WriteFldAlign(BL_CONTROL_MAIN_04, 1, DBC_IIR_CLR_TIM);
	
    vIO32WriteFldAlign(SBC_PARAM_00, 55, SBC_BL_PSH);
    vIO32WriteFldAlign(SBC_PARAM_01, PANEL_GetBacklightHigh(), SBC_BL_TOP);
    vIO32WriteFldAlign(SBC_PARAM_01, PANEL_GetBacklightMiddle(), SBC_BL_MIDDLE);
    vIO32WriteFldAlign(SBC_PARAM_01, PANEL_GetBacklightLow(), SBC_BL_BOTTOM);
    vIO32WriteFldAlign(SBC_PARAM_01, 36, SBC_BL_SETUDEN);
	vIO32WriteFldAlign(SBC_PARAM_02, 15, SBC_BL_PSH_CARBAT);
	vIO32WriteFldAlign(SBC_PARAM_02, 55, SBC_BL_PSH_NORM);
	vIO32WriteFldAlign(SBC_PARAM_02, 29, SBC_BL_PSH_PWRBANK);
    
    //-- Patch for UC 48FHD and 32W panel
    #ifdef DBC_MODEL_US
    if(((FlashPanel_GetInchSize()==32)&&(PANEL_GetPanelWidth() <= 1440))||
        (FlashPanel_GetInchSize()==48))
    {
        vIO32WriteFldAlign(SBC_PARAM_01, 165, SBC_BL_MIDDLE);
    }
    #endif
    //-- End Patch for UC 48FHD and 32W panel

    //Power Saving part
    vIO32WriteFldAlign(DBC_PS_PARAM_00, 65, DBC_PS_LUMA_TH_DIG);
    vIO32WriteFldAlign(DBC_PS_PARAM_00, 70, DBC_PS_LUMA_TH_ANA);
    vIO32WriteFldAlign(DBC_PS_PARAM_00, 64, DBC_CRI_LUMA_TH);
    vIO32WriteFldAlign(DBC_PS_PARAM_00, 0,  DBC_PS_SAT_TH);
    vIO32WriteFldAlign(DBC_PS_PARAM_01, 72, DBC_PS_TGT);
    vIO32WriteFldAlign(DBC_PS_PARAM_01, 81, DBC_PS_TM_ENTER);
    vIO32WriteFldAlign(DBC_PS_PARAM_01, 0,  DBC_PS_TM_EXIT);
    vIO32WriteFldAlign(DBC_PS_PARAM_02, 4,  DBC_PS_ATN_DOWN);
    vIO32WriteFldAlign(DBC_PS_PARAM_02, 20, DBC_PS_ATN_UP);

    //Zero-D dimming
    vIO32WriteFldAlign(ZEROD_DIMMING_PARAM, 255, ZEROD_HEAT_BLSAVE);
    vIO32WriteFldAlign(ZEROD_DIMMING_PARAM, 128, ZEROD_HEAT_BLSAVE_GAIN);

    //Blink part
    vIO32WriteFldAlign(BLINK_PARAM, 127, BLINK_GAIN_TGT);
    vIO32WriteFldAlign(BLINK_PARAM, 0, BLINK_PHASE);
    vIO32WriteFldAlign(BLINK_PARAM, 50, BLINK_DUTY);    //Duty 50%

    //DBC IIR Param part
    vIO32WriteFldMulti(DBC_PARAM0_APL_00,P_Fld(50, DBCP_APL_IIR_TH1)
                                          |P_Fld(100, DBCP_APL_IIR_TH2)
                                          |P_Fld(150, DBCP_APL_IIR_TH3)
                                          |P_Fld(200, DBCP_APL_IIR_TH4));
    vIO32WriteFldMulti(DBC_PARAM0_APL_01,P_Fld(64, DBCP_APL_IIR_L)
                                          |P_Fld(160, DBCP_APL_IIR_M)
                                          |P_Fld(255, DBCP_APL_IIR_H));
    vIO32WriteFldMulti(DBC_PARAM0_BK_00,P_Fld(50, DBCP_BK_IIR_TH1)
                                          |P_Fld(100, DBCP_BK_IIR_TH2)
                                          |P_Fld(150, DBCP_BK_IIR_TH3)
                                          |P_Fld(200, DBCP_BK_IIR_TH4));
    vIO32WriteFldMulti(DBC_PARAM0_BK_01, P_Fld(64, DBCP_BK_IIR_L)
                                          |P_Fld(160, DBCP_BK_IIR_M)
                                          |P_Fld(255, DBCP_BK_IIR_H));
    vIO32WriteFldMulti(DBC_PARAM0_MAX_DET_00,P_Fld(50, DBCP_MAX_DET_IIR_TH1)
                                          |P_Fld(100, DBCP_MAX_DET_IIR_TH2)
                                          |P_Fld(150, DBCP_MAX_DET_IIR_TH3)
                                          |P_Fld(200, DBCP_MAX_DET_IIR_TH4));
    vIO32WriteFldMulti(DBC_PARAM0_MAX_DET_01,P_Fld(64, DBCP_MAX_DET_IIR_L)
                                          |P_Fld(160, DBCP_MAX_DET_IIR_M)
                                          |P_Fld(255, DBCP_MAX_DET_IIR_H)
                                          |P_Fld(100, DBCP_MAX_DET_TH));
    vIO32WriteFldMulti(DBC_PARAM0_GFX_00, P_Fld(50, DBCP_GFX_IIR_TH1)
                                          |P_Fld(100, DBCP_GFX_IIR_TH2)
                                          |P_Fld(150, DBCP_GFX_IIR_TH3)
                                          |P_Fld(200, DBCP_GFX_IIR_TH4));
    vIO32WriteFldMulti(DBC_PARAM0_GFX_01, P_Fld(64, DBCP_GFX_IIR_L)
                                          |P_Fld(160, DBCP_GFX_IIR_M)
                                          |P_Fld(255, DBCP_GFX_IIR_H)
                                          |P_Fld(255, DBCP_GFX_TGT));

    //DBC Calc Param part
    vIO32WriteFldAlign(DBC_PARAM1_APL_00, 1, DBC_APL_EN);
    vIO32WriteFldAlign(DBC_PARAM1_BK_00,  1, DBC_BK_EN);
    vIO32WriteFldAlign(DBC_PARAM1_MAX_DET_00, 1, DBC_MAX_DET_EN);
    vIO32WriteFldAlign(DBC_PARAM1_GFX_00, 1, DBC_GFX_EN);
    
    vIO32WriteFldMulti(DBC_PARAM1_APL_01,P_Fld(80, DBC_APL_TH0)
                                          |P_Fld(160, DBC_APL_TH1)
                                          |P_Fld(40, DBC_APL_TGT));
    
    vIO32WriteFldMulti(DBC_PARAM1_BK_01,P_Fld(80, DBC_BK_TH0)
                                          |P_Fld(160, DBC_BK_TH1)
                                          |P_Fld(40, DBC_BK_TGT));
    
    vIO32WriteFldMulti(DBC_PARAM1_MAX_DET_01,P_Fld(80, DBC_MAX_DET_TH0)
                                          |P_Fld(160, DBC_MAX_DET_TH1));

    //EEI Part
    #if defined(DBC_EEI_CHD) || defined(DBC_EEI_US) || defined(DBC_EEI_EU) \
        || defined(DBC_EEI_PAA) || defined(DBC_EEI_PAD)
    vIO32WriteFldAlign(EEI_PARAM_00, 1, DBC_EEI_ON);
    #else
    vIO32WriteFldAlign(EEI_PARAM_00, 0, DBC_EEI_ON);
    #endif
    
    #ifdef DBC_EEI_CHD
    vIO32WriteFldAlign(EEI_PARAM_00, 0x25, DBC_EEI_INCREASE);
    vIO32WriteFldAlign(EEI_PARAM_01, 0x164, DBC_EEI_DARK_TH);
    vIO32WriteFldAlign(EEI_PARAM_01, 0x32, DBC_EEI_WHITE_TH);
    #else 
    //#if defined(DBC_EEI_US) || defined(DBC_EEI_EU) || defined(DBC_EEI_PAA) || defined(DBC_EEI_PAD)
    vIO32WriteFldAlign(EEI_PARAM_00, 0x30, DBC_EEI_INCREASE);
    vIO32WriteFldAlign(EEI_PARAM_01, 0x90, DBC_EEI_DARK_TH);
    vIO32WriteFldAlign(EEI_PARAM_01, 0xDF, DBC_EEI_WHITE_TH);
    //#endif
    #endif
    
    vIO32WriteFldAlign(EEI_PARAM_00, 0x7, DBC_EEI_STEP);

    //PCC  -- Init PCC related IIR weighting
    vIO32WriteFldAlign(DBC_PCC_00, 50, BL_POWER_LIMIT_IIRP);
    vIO32WriteFldAlign(DBC_PCC_00, 50, BL_POWER_LIMIT_IIRM);
    
    DbcParam_IIR[E_DBC_APL].u1Result=0;
    DbcParam_IIR[E_DBC_BK].u1Result=0;
    DbcParam_IIR[E_DBC_MAX_DET].u1Result=0;
    DbcParam_IIR[E_DBC_GFX].u1Result=0;

    bApiFlashPqUpdateRawDataEx(1, 0);//update IIR param from pq.bin

    _vDRVCUST_DBC_Reg_Update();
    _vDRVCUST_DBC_IIR_Param_Update();
    _vDRVCUST_DBC_Calc_Param_Update();
}

void DRVCUST_AdaptiveBacklightProc(void)
{
    // ------------------------------------------------------------------------------------------------------------
    // this function defines customized algorithm for backlight behavior, which may contain
    // dynamic backlight, light sensor control, etc
    //
    // following listed function are frequently used in these algorithms, you could also find them
    // in drv_meter.c :
    // 
    // UINT8 bDrvGetAPL(void) : returns average picture level
    // UINT8 u1DrvGetLumaHist(UINT16 waHistCur[LUMA_HIST_NUM], UINT16 u2NormBase) : get (normalized) luma histogram
    // UINT8 bDrvGetOutputLumaMax(UINT8 bPath) : returns picture luma maximum value in output stage
    //
    // you should set your algorithm result through DRVCUST_SetBacklightDuty(UINT8 u1RefIndex, UINT16 u2Duty)
    // that you can manipulate final backlight result that may controlled by several backlight algorithms
    //
    // sample code example : 
    //
    // UINT8 u1CurrentAPL = bDrvGetAPL();
    // DRVCUST_SetBacklightDuty(VDP_ARG_BACKLT_DBL, u1CurrentAPL);  // which directly maps backlight from APL        
    // ------------------------------------------------------------------------------------------------------------
	UINT8 u1PWR_Type;
	static BOOL _bPreTorchLightOnOff = 0xFF;//just for init value
	static UINT8 u1Pre_PWR_Type=0xEE;//just for init value
	static UINT8  u1PreUI_PS_Mode=0xFF;
	
	u1PWR_Type = SelfDiagGetPowerMode();//IO32ReadFldAlign(DBC_PARAM1_MAIN, DBC_POWER_BANK_PWR_TYPE);

    DBC_Reg.u1BL_UPDATE_EN = IO32ReadFldAlign(BL_CONTROL_MAIN_00, BL_UPDATE_EN);
    
    // add for picture off on backlight sequence
	if(bPICBACKLTONOFF == SV_OFF)
	{
		DRVCUST_SetBacklightDuty(VDP_ARG_BACKLT_DBL, (UINT8)0x0); // add for dimming can't set zero when pictureoff for time issue.
        vDRVCUST_BLSet_IP_REF(0);
        DRVCUST_SetScanPWM_High_Low(0);
        return;
	}
    else if(bPICBACKLTONOFF == SV_ON)
	{
		x_thread_delay(1);
		bPICBACKLTONOFF=0xFF;
	}                               

    //---Main Control---
    if (DBC_Reg.u1BL_UPDATE_EN == SV_TRUE)
    {
        _vDRVCUST_DBLProc();
    }
	////////////////////////////torchlight
	if((DBC_Reg.u1UI_PS_Mode != u1PreUI_PS_Mode)||(u1PWR_Type!=u1Pre_PWR_Type)||(_bTorchLightOnOff!=_bPreTorchLightOnOff))
    {
        DBC_Reg.u1UI_BL_Level = IO32ReadFldAlign(BL_CONTROL_MAIN_01, UI_BL_LEVEL);
        DBC_Reg.u1UI_PS_Mode = IO32ReadFldAlign(BL_CONTROL_MAIN_01, UI_BL_PS_MODE);
        printf("zdo in u1UI_PS_Mode=%d\n",(DBC_Reg.u1UI_PS_Mode));
        printf("zdo in u1PWR_Type=%d\n",(u1PWR_Type));
        printf("zdo in u1Pre_PWR_Type=%d\n",(u1Pre_PWR_Type));
        printf("zdo in _bTorchLightOnOff=%d\n",(_bTorchLightOnOff));
		if(u1PWR_Type==2)
		{
			if(DBC_Reg.u1UI_PS_Mode==3)
			{
				if(_bTorchLightOnOff==SV_OFF)
				{
					GPIO_SetOut(57,1);
					printf("zdo GPIO ~~~1\n");
				}
				else
				{
					GPIO_SetOut(57,0);
					printf("zdo GPIO ~~~0\n");
				}
				
			}
			else
			{
					GPIO_SetOut(57,0);
					printf("zdo GPIO ~------------------------------~~0\n");
			}


			}
		else
		{
				GPIO_SetOut(57,0);
				printf("zdo GPIO ++++~~~0\n");
		}
      }

	u1PreUI_PS_Mode= DBC_Reg.u1UI_PS_Mode;
    u1Pre_PWR_Type= u1PWR_Type;
    _bPreTorchLightOnOff= _bTorchLightOnOff;
	////////////////////////////
    //--- LED Motion ---
    if ((vDrvGetLCDFreq() == 25) || (vDrvGetLCDFreq() == 50) || (vDrvGetLCDFreq() == 100))  // 50Hz
    {
        u1Is50HZ = SV_TRUE;
    }
    else
    {
        u1Is50HZ = SV_FALSE;
    }

    if ((u1preIs50HZ != u1Is50HZ) 
        || (DRVCUST_GetLEDMotionFlag() != IO32ReadFldAlign(BLINK_PARAM,LEDMOTION_MODE)))
    {
        u1preIs50HZ = u1Is50HZ;
		DRVCUST_SetLEDMotion(DRVCUST_GetLEDMotionFlag());
        vIO32WriteFldAlign(BL_CONTROL_MAIN_04,BACKLT_GetDimmingFrequency(), BL_PWM_FRQ);    //update FRQ information
    }
}

