/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/** @file pinmux.h
 *  pinmux.h declare all pinmux/gpio private related function and parameters.
 */

#ifndef FViperPINMUX_H
#define FViperPINMUX_H

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "x_lint.h"
#include "hw_ckgen.h"
#include "x_ckgen.h"
#include "x_gpio.h"

//=====================================================================
// IC information
#ifndef CC_GPIO_MCM
#define TOTAL_PIN_REG           (8)
#else
#define TOTAL_PIN_REG           (10)

#endif
 
//=====================================================================
// Register definitions
#ifndef CC_GPIO_MCM
#define MX0	((1U << 5) | BASIC_MASK) //MX0
#define MX1	((2U << 5) | BASIC_MASK) //MX1
#define MX2	((3U << 5) | BASIC_MASK) //MX2
#define MX3	((4U << 5) | BASIC_MASK) //MX3
#define MX4	((5U << 5) | BASIC_MASK) //MX4
#define MX5	((6U << 5) | BASIC_MASK) //MX5
#define PDMX0	((7U << 5) | BASIC_MASK) //PDMX0
#define PDMX1	((8U << 5) | BASIC_MASK) //PDMX1
#define PMISC MX5

#else

#define MX0	((1U << 5) | BASIC_MASK) //MX0
#define MX1	((2U << 5) | BASIC_MASK) //MX1
#define MX2	((3U << 5) | BASIC_MASK) //MX2
#define MX3	((4U << 5) | BASIC_MASK) //MX3
#define MX4	((5U << 5) | BASIC_MASK) //MX4
#define MX5	((6U << 5) | BASIC_MASK) //MX5
#define MX6	((7U << 5) | BASIC_MASK) //MX6
#define MX7	((8U << 5) | BASIC_MASK) //MX7
#define PDMX0	((9U << 5) | BASIC_MASK) //PDMX0
#define PDMX1	((10U << 5) | BASIC_MASK) //PDMX1
#define PMISC MX7


#endif

#define PIN_OSDA0   (MX0|OFFSETMASK(1,0))
#define PIN_OSCL0   (MX0|OFFSETMASK(1,0))
#define PIN_GPIO0   (MX0|OFFSETMASK(4,2))
#define PIN_GPIO1   (MX0|OFFSETMASK(7,5))
#define PIN_GPIO2   (MX0|OFFSETMASK(10,8))
#define PIN_GPIO3   (MX0|OFFSETMASK(13,11))
#define PIN_GPIO4   (MX0|OFFSETMASK(16,14))
#define PIN_GPIO5   (MX0|OFFSETMASK(19,17))
#define PIN_GPIO6   (MX0|OFFSETMASK(22,20))
#define PIN_GPIO7   (MX0|OFFSETMASK(25,23))
#define PIN_GPIO8   (MX0|OFFSETMASK(28,26))

#define PIN_GPIO10   (MX4|30)
#define PIN_GPIO9   (MX0|OFFSETMASK(31,29))
#define PIN_GPIO11   (MX4|31)
#define PIN_POCE1_   (MX1|1)
#define PIN_PDD0   (MX1|0)
#define PIN_PDD1   (MX1|0)
#define PIN_PDD2   (MX1|OFFSETMASK(5,4))
#define PIN_PDD3   (MX1|OFFSETMASK(5,4))
#define PIN_PDD4   (MX1|OFFSETMASK(5,4))
#define PIN_PDD5   (MX1|OFFSETMASK(5,4))
#define PIN_PDD6   (MX1|OFFSETMASK(5,4))
#define PIN_PDD7   (MX1|OFFSETMASK(5,4))
#define PIN_PARB_   (MX1|3)
#define PIN_POOE_   (MX1|0)
#define PIN_POCE0_   (MX1|0)
#define PIN_PACLE   (MX1|3)
#define PIN_PAALE   (MX1|3)
#define PIN_POWE_   (MX1|3)
#define PIN_ALIN   (MX1|OFFSETMASK(7,6))
#define PIN_ASPDIFO0   (MX1|OFFSETMASK(9,8))
#define PIN_AOMCLK   (MX1|OFFSETMASK(11,10))
#define PIN_AOLRCK   (MX1|OFFSETMASK(11,10))
#define PIN_AOBCK   (MX1|OFFSETMASK(11,10))
#define PIN_AOSDATA0   (MX1|OFFSETMASK(11,10))
#define PIN_AOSDATA1   (MX1|OFFSETMASK(13,12))
#define PIN_HDMI_CEC   (PDMX1|OFFSETMASK(1,0))

#define PIN_HDMI_1_SCL   (PDMX1|OFFSETMASK(3,2))
#define PIN_HDMI_1_SDA   (PDMX1|OFFSETMASK(5,4))
#define PIN_HDMI_1_HPD   (PDMX1|OFFSETMASK(7,6))
#define PIN_HDMI_2_SCL   (PDMX1|OFFSETMASK(9,8))
#define PIN_HDMI_2_SDA   (PDMX1|OFFSETMASK(11,10))
#define PIN_HDMI_2_HPD   (PDMX1|OFFSETMASK(13,12))
#define PIN_HDMI_3_SCL   (PDMX1|OFFSETMASK(15,14))
#define PIN_HDMI_3_SDA   (PDMX1|OFFSETMASK(17,16))
#define PIN_HDMI_3_HPD   (PDMX1|OFFSETMASK(19,18))
#define PIN_OPCTRL0   (PDMX0|OFFSETMASK(2,0))
#define PIN_OPCTRL1   (PDMX0|OFFSETMASK(5,3))
#define PIN_OPCTRL2   (PDMX0|OFFSETMASK(8,6))
#define PIN_OPCTRL3   (PDMX0|OFFSETMASK(11,9)| MULTI_GPIO_FLAG)
#define PIN_OPCTRL4   (PDMX0|OFFSETMASK(21,19)| MULTI_GPIO_FLAG)
#define PIN_VGA_SCL   (PDMX0|OFFSETMASK(14,12))
#define PIN_VGA_SDA   (PDMX0|OFFSETMASK(14,12))
#define PIN_U0RX   (PDMX0|OFFSETMASK(18,16))
#define PIN_U0TX   (PDMX0|OFFSETMASK(18,16))
#define PIN_OIRI   (PDMX0|OFFSETMASK(25,24))
#define PIN_OPWRSB   (PDMX0|OFFSETMASK(23,22))
#define PIN_ADIN5   (PDMX0|31)
#define PIN_ADIN4   (PDMX0|30)
#define PIN_ADIN3   (PDMX0|29)
#define PIN_ADIN2   (PDMX0|28)
#define PIN_AIN1_L   (MX2|31)
#define PIN_AIN2_L   (MX2|31)
#define PIN_AIN3_L   (MX2|31)
#define PIN_AIN4_L   (MX2|31)
#define PIN_AIN1_R   (MX2|31)
#define PIN_AIN2_R   (MX2|31)
#define PIN_AIN3_R   (MX2|31)
#define PIN_AIN4_R   (MX2|31)
#define PIN_AL0   (MX3|31)
#define PIN_AL1   (MX3|31)
#define PIN_AL2   (MX3|31)
#define PIN_AR0   (MX3|31)
#define PIN_AR1   (MX3|31)
#define PIN_AR2   (MX3|31)
#define PIN_USB_DM_P0   (MX1|OFFSETMASK(15,14))
#define PIN_USB_DP_P0   (MX1|OFFSETMASK(15,14))
#define PIN_USB_DM_P1   (MX1|OFFSETMASK(17,16))
#define PIN_USB_DP_P1   (MX1|OFFSETMASK(17,16))
#define PIN_CIGPIO0   (MX5|OFFSETMASK(2,0))
#define PIN_CIGPIO1   (MX5|OFFSETMASK(2,0))
#define PIN_CIGPIO2   (MX5|OFFSETMASK(2,0))
#define PIN_CIGPIO3   (MX5|OFFSETMASK(2,0))
#define PIN_CIGPIO4   (MX5|OFFSETMASK(5,3))
#define PIN_CIGPIO5   (MX5|OFFSETMASK(5,3))
#define PIN_CIGPIO6   (MX5|OFFSETMASK(5,3))
#define PIN_CIGPIO7   (MX5|OFFSETMASK(5,3))
#define PIN_CIGPIO8   (MX5|OFFSETMASK(8,6))
#define PIN_CIGPIO9   (MX5|OFFSETMASK(8,6))
#define PIN_CIGPIO10   (MX5|OFFSETMASK(8,6))
#define PIN_CIGPIO11   (MX5|OFFSETMASK(10,9))
#define PIN_CIGPIO12   (MX5|OFFSETMASK(10,9))
#define PIN_CIGPIO13   (MX5|OFFSETMASK(10,9))
#define PIN_CIGPIO14   (MX5|OFFSETMASK(10,9))
#define PIN_CIGPIO15   (MX5|OFFSETMASK(12,11))
#define PIN_CIGPIO16   (MX5|OFFSETMASK(12,11))
#define PIN_CIGPIO17   (MX5|OFFSETMASK(12,11))
#define PIN_CIGPIO18   (MX5|OFFSETMASK(12,11))
#define PIN_CIGPIO19   (MX5|OFFSETMASK(15,13))
#define PIN_CIGPIO20   (MX5|OFFSETMASK(17,16))
#define PIN_CIGPIO21   (MX5|OFFSETMASK(17,16))
#define PIN_CIGPIO22   (MX5|OFFSETMASK(17,16))
#define PIN_CIGPIO23   (MX5|OFFSETMASK(19,18))
#define PIN_CIGPIO24   (MX5|OFFSETMASK(19,18))
#define PIN_CIGPIO25   (MX5|OFFSETMASK(19,18))
#define PIN_CIGPIO26   (MX5|OFFSETMASK(21,20))
#define PIN_CIGPIO27   (MX5|OFFSETMASK(21,20))
#define PIN_CIGPIO28   (MX5|OFFSETMASK(21,20))
#define PIN_CIGPIO29   (MX5|OFFSETMASK(21,20))
#define PIN_CIGPIO30   (MX5|OFFSETMASK(24,22))
#define PIN_CIGPIO31   (MX5|OFFSETMASK(27,25))
#define PIN_CIGPIO32   (MX5|OFFSETMASK(30,28))
#define PIN_CIGPIO33   (MX5|OFFSETMASK(30,28))
#define PIN_CIGPIO34   (MX2|OFFSETMASK(6,4)| MULTI_GPIO_FLAG)
#define PIN_CIGPIO35   (MX2|OFFSETMASK(6,4)| MULTI_GPIO_FLAG)
#define PIN_CIGPIO36   (MX2|OFFSETMASK(9,7))
#define PIN_CIGPIO37   (MX2|OFFSETMASK(11,10))
#define PIN_CIGPIO38   (MX2|OFFSETMASK(11,10))
#define PIN_CIGPIO39   (MX2|OFFSETMASK(11,10))
#define PIN_CIGPIO40   (MX2|OFFSETMASK(11,10))
#define PIN_CIGPIO41   (MX2|OFFSETMASK(13,12))
#define PIN_CIGPIO42   (MX2|OFFSETMASK(15,14))
#define PIN_CIGPIO43   (MX2|OFFSETMASK(15,14))
#define PIN_CIGPIO44   (MX2|OFFSETMASK(15,14))
#define PIN_CIGPIO45   (MX2|OFFSETMASK(17,16)| MULTI_GPIO_FLAG)
#define PIN_CIGPIO46   (MX2|OFFSETMASK(17,16)| MULTI_GPIO_FLAG)
#define PIN_CIGPIO47   (MX2|OFFSETMASK(17,16))
#define PIN_CIGPIO48   (MX2|OFFSETMASK(19,18)| MULTI_GPIO_FLAG)
#define PIN_CIGPIO49   (MX2|OFFSETMASK(21,20))
#define PIN_CIGPIO50   (MX2|OFFSETMASK(23,22))
#define PIN_CIGPIO51   (MX2|OFFSETMASK(25,24))
#define PIN_CIGPIO52   (MX2|OFFSETMASK(25,24))
#define PIN_CIGPIO53   (MX2|OFFSETMASK(25,24))
#define PIN_CIGPIO54   (MX2|OFFSETMASK(25,24))
#define PIN_CIGPIO55   (MX2|OFFSETMASK(25,24))
#define PIN_CIGPIO56   (MX2|OFFSETMASK(25,24))
#define PIN_IF_AGC   (MX2|26)
#define PIN_RF_AGC   (MX2|27)
#define PIN_OSDA2   (MX2|OFFSETMASK(30,28))
#define PIN_OSCL2   (MX2|OFFSETMASK(30,28))
#define PIN_OPWM0   (MX3|OFFSETMASK(2,0)| MULTI_GPIO_FLAG)
#define PIN_OPWM1   (MX3|OFFSETMASK(5,3)| MULTI_GPIO_FLAG)
#define PIN_OPWM2   (MX3|OFFSETMASK(8,6))
#define PIN_OPWM3   (MX3|OFFSETMASK(11,9)| MULTI_GPIO_FLAG)
#define PIN_OPWM4   (MX3|OFFSETMASK(14,12))
#define PIN_OPWM5   (MX3|OFFSETMASK(17,15))
#define PIN_OPWM6   (MX3|OFFSETMASK(20,18))
#define PIN_TCON0   (MX3|OFFSETMASK(23,21))
#define PIN_TCON1   (MX3|OFFSETMASK(26,24))
#define PIN_TCON2   (MX3|OFFSETMASK(29,27))
#define PIN_TCON3   (MX4|OFFSETMASK(2,0))
#define PIN_TCON4   (MX4|OFFSETMASK(5,3))
#define PIN_TCON5   (MX4|OFFSETMASK(8,6))
#define PIN_TCON6   (MX4|OFFSETMASK(11,9))
#define PIN_TCON7   (MX4|OFFSETMASK(14,12))
#define PIN_TCON8   (MX4|OFFSETMASK(17,15))
#define PIN_TCON9   (MX4|OFFSETMASK(20,18))
#define PIN_TCON10   (MX4|OFFSETMASK(23,21))
#define PIN_TCON11   (MX4|OFFSETMASK(26,24))
#define PIN_TCON12   (MX4|OFFSETMASK(29,27))

#ifdef CC_GPIO_MCM
#define PIN_MCM_XTAL   (MX6|OFFSETMASK(2,0))
#define PIN_MCM_GPIO1   (MX6|OFFSETMASK(5,3))
#define PIN_MCM_GPIO2   (MX6|OFFSETMASK(8,6))
#define PIN_MCM_GPIO3   (MX6|OFFSETMASK(11,9))
#define PIN_MCM_GPIO4   (MX6|OFFSETMASK(14,12))
#define PIN_MCM_GPIO5   (MX6|OFFSETMASK(17,15))
#define PIN_MCM_GPIO6   (MX6|OFFSETMASK(20,18))
#define PIN_MCM_GPIO7   (MX6|OFFSETMASK(23,21))
#define PIN_MCM_GPIO8   (MX6|OFFSETMASK(26,24))
#define PIN_MCM_GPIO9   (MX6|OFFSETMASK(29,27))
#define PIN_MCM_GPIO10   (MX7|OFFSETMASK(2,0))
#define PIN_MCM_GPIO11   (MX7|OFFSETMASK(5,3))
#define PIN_MCM_GPIO12   (MX7|OFFSETMASK(8,6))
#define PIN_USB_DM_P2   (MX7|9)
#define PIN_USB_DP_P2   (MX7|10)

#endif 
 // gpio pin without pinmux
#define PIN_NO_PINMUX       (PMISC | 31)
#define PIN_LAST_PINMUX     (0xffffffff)

 // backward compatible
#define PINMUX_OPCTRL_0         PIN_OPCTRL0

//-----------------------------------------------------------------------------
// Macro definitions (IC independant)
//-----------------------------------------------------------------------------
#define PINMUX_FUNCTION0        0
#define PINMUX_FUNCTION1        1
#define PINMUX_FUNCTION2        2
#define PINMUX_FUNCTION3        3
#define PINMUX_FUNCTION4        4
#define PINMUX_FUNCTION5        5
#define PINMUX_FUNCTION6        6
#define PINMUX_FUNCTION7        7
#define PINMUX_FUNCTION8        8
#define PINMUX_FUNCTION9        9
#define PINMUX_FUNCTION10       10
#define PINMUX_FUNCTION11       11
#define PINMUX_FUNCTION12       12
#define PINMUX_FUNCTION13       13
#define PINMUX_FUNCTION14       14
#define PINMUX_FUNCTION15       15
 
#define IS_VALID_MUX_REG(pin)   (((pin) & (0x0fU << 5)) != 0)
#define OFFSETMASK(ebit, sbit)  ((BITNUM_TO_MASK((ebit) - (sbit)) << 12) | (sbit))
#define MUXHIB(x)               ((x) << 16)
#define MAX_FUNC(pin)           (LO_MAX_FUNC(pin) | (HI_MAX_FUNC(pin) << (MASK_TO_BITNUM(LO_MAX_FUNC(pin))+1)))
#define BITNUM_TO_MASK(m)       (((m) > 1) ? (((m)==2) ? 7 : 15) : (((m)==1) ? 3 : 1))
#define MASK_TO_BITNUM(m)       (((m) > 3) ? (((m)==7) ? 2 : 3) : (((m)==1) ? 0 : 1))
#define MULTI_GPIO_FLAG         (1U << 10)
#define BASIC_MASK              (1U << 12)
 
#define LO_BIT_OFFSET(pin)      ((pin >> 0) & 0x1fU)
#define LO_MUX_REG(pin)         (((pin) >> 5) & 0x0fU)
#define LO_MAX_FUNC(pin)        (((pin) >> 12) & 0x0fU)
#define LO_MASK(pin)            (LO_MAX_FUNC(pin))
#define HI_BIT_OFFSET(pin)      (((pin) >> 16) & 0x1fU)
#define HI_MUX_REG(pin)         (((pin) >> 21) & 0x0fU)
#define HI_MAX_FUNC(pin)        (((pin) >> 28) & 0x0fU)
#define HI_MASK(pin)            (HI_MAX_FUNC(pin))
 
//-----------------------------------------------------------------------------
// Public function declaration
//-----------------------------------------------------------------------------

/* Pinmux function. */
extern INT32 BSP_PinSet(UINT32 u4PinSel, INT32 i4Func);
extern INT32 BSP_PinGet(UINT32 u4PinSel);
extern INT32 BSP_PinGpioSel(INT32 i4Gpio);
extern INT32 BSP_PinGpioGet(INT32 i4Gpio, UINT32 *pu4PinSel);

#endif // X_PINMUX_H

