###############################################################################
#
# This file is provided under a dual license.  When you use or
# distribute this software, you may choose to be licensed under
# version 2 of the GNU General Public License ("GPLv2 License")
# or BSD License.
#
# GPLv2 License
#
# Copyright(C) 2019 MediaTek Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of version 2 of the GNU General Public License as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See http://www.gnu.org/licenses/gpl-2.0.html for more details.
#
# BSD LICENSE
#
# Copyright(C) 2019 MediaTek Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
###############################################################################
###########################################################################
# $RCSfile: customer.mak,v $
# $Revision: #6 $
# $Date: 2015/06/11 $
# $Author: mark.hu $
# $MD5HEX: 61118bc277d8ac078b83c14a0d509c1a $
#
# Description:
#         Makefile to build a "MT-5351" demo board build. The following
#         targets are supported:
#
#             all:           Compiles middleware/open, custom, target and this
#                            directory and creates a final image.
#############################################################################




#
#Define showstring instead of warning
#
ifeq (1,$(V))
define showstring
$(warning $(1))
endef
else
define showstring
endef
endif

#
# Set the Use Script
#

RM      = /bin/rm
RM_FLAG = -rf

RMDIR   = /bin/rmdir
RMDIR_FLAG = --ignore-fail-on-non-empty -p

LN      = /bin/ln
LN_FLAG = -sf

CP      = /bin/cp
CP_FLAG = -rf
RLS_CP_FLAG = -au


MV      = /bin/mv
MV_FLAG =


export FCLEAN = $(TOOL_ROOT)/source/open/fast_clean/fclean.sh
export SHELL = /bin/bash

#
# Make command and options
#
MAKE      = make SHELL=$(SHELL)
MAKE_FLAG =
export JOBS  ?= $(shell cat /proc/cpuinfo |grep processor|wc -l)
FMAKE     = $(MAKE) -j $(JOBS)


SUPPORT_MAKE_VERSION := 3.81
MAKE_VERSION := $(shell make -v|grep 'GNU Make'|awk '{print $$3}')

# check for broken versions of make
ifeq (0,$(shell expr $(MAKE_VERSION) \>= 3.81))
$(warning ********************************************************************************)
$(warning *  You are using version $(MAKE_VERSION) of make.)
$(warning *  You must upgrade to version $(SUPPORT_MAKE_VERSION) or greater.)
$(warning ********************************************************************************)
$(error stopping)
endif


#
# Create directory command and options
#
MKDIR      = mkdir
MKDIR_FLAG = -p

#
# tar command and options
#
TAR        = tar
TAR_FLAG   = zcf
UNTAR_FLAG = zxf
UNTAR_BZ_FLAG = jxf


#
# gzip command and options
#
GZIP        = gzip


#
# find command and options
#
FIND        = /usr/bin/find

REMOVE_SH := $(CHILING_ROOT)/script/base/_remove_sub.sh
RELEASE_DRIVER_SCRIPT     := $(CHILING_ROOT)/script/base/_release_driver.sh
RELEASE_DRIVER_OBJ_SCRIPT := $(CHILING_ROOT)/script/base/_release_driver_obj.sh

#
# Date command and options
#
DATE = date
DATE_FLAG = +%Y%m%d_%H%M%S
SDATE_FLAG = +%s

#
# file/directory change mode command
#
CHMOD      = /bin/chmod

LOCKFILE = lockfile -1 -l 30

#
# The shell define
#
export SHELL=/bin/bash

#
# mtk tool chain
#
export GENEXT2FS       = $(IMAGE_ROOT_PATH)/genext2fs
export MKCRAMFS        = $(IMAGE_ROOT_PATH)/mkcramfs
export MKIMAGE         = $(IMAGE_ROOT_PATH)/mkimage
export MKSQUASHFS      = $(IMAGE_ROOT_PATH)/mksquashfs$(MKFS_VERSION)
# mksquashfs4.3: selinux+xattr usage
export SE_MKSQUASHFS   = $(IMAGE_ROOT_PATH)/mksquashfs4.3
export PACKLINUX       = $(IMAGE_ROOT_PATH)/packlinux
ifeq "$(USE_MKNFSBIMG_V3)" "true"
export MKNFSBIMG       = $(IMAGE_ROOT_PATH)/mknfsbimg3
export NFSBVERITY      = $(IMAGE_ROOT_PATH)/veritysetup
else
export MKNFSBIMG       = $(IMAGE_ROOT_PATH)/mknfsbimg
export NFSBVERITY      = $(IMAGE_ROOT_PATH)/verity
endif

export MTK_MKIMAGE := $(MKIMAGE)
  
export GEN_CFG := $(TOOL_ROOT)/binary/open/gen_cfg/gen_cfg_linux
export CFG_PARSER  = $(TOOL_ROOT)/binary/open/parse_sort_cfg/parse_sort_cfg
#
# The following functions are used to check on files and execute
# a set of function.
#

ifndef SYMLINK_LOG
export SYMLINK_LOG := /dev/null
endif

if_file_notexist_w_sym_fct   = if [ -e $(1) -o -h $(1) ]; then if [ ! -e $(2) -a ! -h $(2) ]; then $(LN) $(LN_FLAG) $(1) $(2); if [ "$(SYMLINK_LOG)" != "" ]; then echo  $(2) >> $(SYMLINK_LOG); fi ; fi ; fi

#
# The following functions are used to check on files and execute
# a set of function.
#
if_file_exist_fct     = @if [ -e $(1) ]; then $(2) fi
if_file_not_exist_fct = @if [ ! -e $(1) ]; then $(2) fi


#
# The following functions are used to check on directories and execute
# a set of function.
#
if_dir_exist_fct     = @if [ -d $(1) ]; then $(2) fi
if_dir_not_exist_fct = @if [ ! -d $(1) ]; then $(2) fi


#
# The following functions are used to calculate the execute time of
# a set of function.
#
cal_exec_time_fct     = echo BUILD_$(1)_END_TIME=`$(DATE) $(DATE_FLAG)`;\
                        if [ -e .build_start_time_$(1) ]; then expr `$(DATE) $(SDATE_FLAG)` - `cat .build_start_time_$(1)` > dtv_linux_build_time_$(1); \
                                              echo BUILD_$(1)_EXEC_TIME:`cat  dtv_linux_build_time_$(1)`; \
                                              rm -rf dtv_linux_build_time_$(1) .build_start_time_$(1); fi
                        
                                              
set_start_time_fct    = $(DATE) $(SDATE_FLAG) > .build_start_time_$(1); \
                        echo BUILD_$(1)_START_TIME=`$(DATE) $(DATE_FLAG)`

export CHK_ERR_WARN_SCRIPT_ROOT  = $(TOOL_ROOT)/source/open/pbuild
export CHK_ERR_WARN_SCRIPT = $(CHK_ERR_WARN_SCRIPT_ROOT)/chk_warn_err.sh 
export ERROR_LOG    = $(THIS_ROOT)/build_fail.log
export BRANCH_NAME  = $(word 2,$(BUILD_NAME))
export FIND_AUTHOR ?= false
export CHECK_WARNING_BUILD ?= false

ifeq "$(RLS_CUSTOM_BUILD)" "true"
export FIND_AUTHOR = false
endif
ifeq "$(CHECK_WARNING_BUILD)" "true"
export FIND_AUTHOR ?= true
endif

export CHECK_WARNING ?= false

chk_err_fct =   sed -e '/\( error:\|first defined here\|multiple definition of\|Stop\.\|No rule to make target\|^make.* Error \)/!d' $(1) > $(1).tmp.txt; \
                if [ -s $(1).tmp.txt ]; then    \
                    grep -v 'error = clazz' $(1).tmp.txt > $(1).tmp2.txt; \
                    if [ -s $(1).tmp2.txt ]; then \
                      mv $(1).tmp2.txt $(1).tmp.txt; \
                    else \
                      rm -rf $(1).tmp.txt; \
                    fi;\
                fi; \
				if [ -s $(1).tmp.txt ]; then	\
					/bin/echo -e '\n\n=========================== BUILD_ERROR_MESSAGES ==========================='; \
					cat $(1).tmp.txt;	\
					rm -f $(1) $(1).tmp.txt;	\
					exit 1;	\
				fi; \
				rm -f $(1) $(1).tmp.txt
          
chk_err_warn_fct     = if [ -e $(1) ]; then /bin/bash $(CHK_ERR_WARN_SCRIPT) $(CHK_ERR_WARN_SCRIPT_ROOT) $(BRANCH_NAME) $(1) $(ERROR_LOG) $(FIND_AUTHOR) $(CHECK_WARNING) $(2);  if [ -e $(ERROR_LOG) ]; then  exit 1; else $(call chk_err_fct, $(1));	 fi ; fi
                                              

cust_link = \
	if [ -d $(1) -a -d $(2) ];  then \
		for i in `cd $(1); find . -type f | sed  's/\.\///'` ; do  \
			if [ -f $(2)/$$i -a ! -L $(2)/$$i ] ; then  \
				ORIG_SRC_FILE=`echo $(2)/$$i | sed 's,$(VM_LINUX_ROOT),$(CUST_ORIG_SRC_ROOT),'` ; \
				$(MKDIR) $(MKDIR_FLAG) `dirname $$ORIG_SRC_FILE` ; \
				$(MV) $(MV_FLAG) $(2)/$$i $$ORIG_SRC_FILE ; \
				$(LN) $(LN_FLAG) $(1)/$$i  $(2)/$$i ; \
				echo $(2)/$$i >> $(CUST_LINK_LOG) ; \
			elif [ ! -e $(2)/$$i	] ; then  \
				$(MKDIR) $(MKDIR_FLAG) `dirname $(2)/$$i ` ; \
				$(LN) $(LN_FLAG) $(1)/$$i  $(2)/$$i ; \
				echo $(2)/$$i >> $(CUST_LINK_LOG) ; \
			fi ; \
		done ; \
	fi

clean_cust_link = \
	if [ -e $(CUST_LINK_LOG) ]; then \
		for i in `cat $(CUST_LINK_LOG) | grep $(1)` ; do \
			$(RM) $(RM_FLAG) $$i; \
			ORIG_SRC_FILE=`echo $$i | sed 's,$(VM_LINUX_ROOT),$(CUST_ORIG_SRC_ROOT),'` ; \
			if [ -e $$ORIG_SRC_FILE ] ; then  \
				$(MV) $(MV_FLAG) $$ORIG_SRC_FILE $$i ; \
			fi ; \
		done; \
	fi
